# -*- coding: utf-8 -*-
import numpy as np
import logging

import matplotlib.pyplot as plt

# pyCGM2
import pyCGM2
from pyCGM2 import enums
from pyCGM2.Report import plot



class AbstractPlotViewer(object):
    """
    Abstract Builder
    """
    def __init__(self,input,AutomaticYlimits=False):
        self.m_input =input
        self.m_automaticYlim_flag = AutomaticYlimits

    def setNormativeData(self):
        pass

    def __setLayer(self):
        pass

    def __setData(self):
        pass

    def plotPanel(self):
        pass

    def setAutomaticYlimits(self,bool):
        self.m_automaticYlim_flag = bool

class SpatioTemporalPlotViewer(AbstractPlotViewer):
    """


    """

    def __init__(self,iAnalysis):

        """
            :Parameters:
                 - `iAnalysis` (pyCGM2.Processing.analysis.Analysis ) - Analysis instance built from AnalysisFilter
        """

        super(SpatioTemporalPlotViewer, self).__init__(iAnalysis)

        self.m_analysis = self.m_input
        if isinstance(self.m_analysis,pyCGM2.Processing.analysis.Analysis):
            pass
        else:
            logging.error( "[pyCGM2] error input object type. must be a pyCGM2.Core.Processing.analysis.Analysis")

    def setNormativeDataset(self,iNormativeDataSet):
        """
            **Description :** set a normative gait dataset

            :Parameters:
                 - `iNormativeDataSet` (a class of the pyCGM2.Report.normativeDataset module) - normative gait dataset from pyCGM2.Report.normativeDatabaseProcedure module

        """
        iNormativeDataSet.constructNormativeData()
        self.m_normativeData = iNormativeDataSet.data

    def __setLayer(self):
        self.fig = plt.figure(figsize=(8.27,11.69), dpi=100,facecolor="white")
        title=u""" Spatio temporal Gait Parameters \n """
        self.fig.suptitle(title)
        plt.subplots_adjust(left=None, bottom=None, right=None, top=None, wspace=0.5, hspace=0.5)

        ax0 = plt.subplot(8,1,1)# speed overall (speed)
        ax1 = plt.subplot(8,1,2)# length overall (strideLength)
        ax2 = plt.subplot(8,1,3)# width overall (strideWidth
        ax3 = plt.subplot(8,1,4)# cadence overall (cadence)

        ax4 = plt.subplot(8,1,5)# Step length  steplength
        ax5 = plt.subplot(8,1,6)# SStep time (s) duration
        ax6 = plt.subplot(8,1,7)# Stance time (% of gait cycle) stancePhase
        ax7 = plt.subplot(8,1,8)# Initial double limb support (s) doubleStance1


    def __setData(self):

        plot.stpHorizontalHistogram(self.fig.axes[0],self.m_analysis.stpStats,
                                "speed",
                                overall= True,
                                title="speed (m.s-1)", xlabel=None,xlim=None)

        plot.stpHorizontalHistogram(self.fig.axes[1],self.m_analysis.stpStats,
                                "strideLength",
                                overall= True,
                                title="stride length (m)", xlabel=None,xlim=None)

        plot.stpHorizontalHistogram(self.fig.axes[2],self.m_analysis.stpStats,
                                "strideWidth",
                                overall= True,
                                title="stride width (m)", xlabel=None,xlim=None)

        plot.stpHorizontalHistogram(self.fig.axes[3],self.m_analysis.stpStats,
                                "cadence",
                                overall= True,
                                title="cadence (strides/min)", xlabel=None,xlim=None)

        plot.stpHorizontalHistogram(self.fig.axes[4],self.m_analysis.stpStats,
                                "stepLength",
                                overall= False,
                                title="Step length (m)", xlabel=None,xlim=None)

        plot.stpHorizontalHistogram(self.fig.axes[5],self.m_analysis.stpStats,
                                "stepDuration",
                                overall= False,
                                title="Step time (s)", xlabel=None,xlim=None)

        plot.stpHorizontalHistogram(self.fig.axes[6],self.m_analysis.stpStats,
                                "stancePhase",
                                overall= False,
                                title="Stance time (% of gait cycle)", xlabel=None,xlim=None)

        plot.stpHorizontalHistogram(self.fig.axes[7],self.m_analysis.stpStats,
                                "doubleStance1",
                                overall= False,
                                title="Initial double limb support (% of gait cycle)", xlabel=None,xlim=None)


    def plotPanel(self):
        self.__setLayer()
        self.__setData()

        if self.m_normativeData is not None:
            labels = ["speed", "stride length", "stride width", "cadence", "step length ","step time", "stance phase","initial double stance"]

            i=0
            for label in labels:
                minusStd = self.m_normativeData[label]["Mean"]-self.m_normativeData[label]["Std"]
                plusStd = self.m_normativeData[label]["Mean"]+self.m_normativeData[label]["Std"]
                self.fig.axes[i].axvline(x=(minusStd), color= "green", linestyle = "dashed")
                self.fig.axes[i].axvline(x=(plusStd), color= "green", linestyle = "dashed")
                i+=1

        return self.fig

class GpsMapPlotViewer(AbstractPlotViewer):
    """


    """

    def __init__(self,iAnalysis,pointLabelSuffix=None):

        """
            :Parameters:
                 - `iAnalysis` (pyCGM2.Processing.analysis.Analysis ) - Analysis instance built from AnalysisFilter
        """

        super(GpsMapPlotViewer, self).__init__(iAnalysis)

        self.m_analysis = self.m_input
        if isinstance(self.m_analysis,pyCGM2.Processing.analysis.Analysis):
            pass
        else:
            logging.error( "[pyCGM2] error input object type. must be a pyCGM2.Core.Processing.analysis.Analysis")

        self.m_pointLabelSuffix = pointLabelSuffix

    def setNormativeDataset(self,iNormativeDataSet):
        """
            **Description :** set a normative gait dataset

            :Parameters:
                 - `iNormativeDataSet` (a class of the pyCGM2.Report.normativeDataset module) - normative gait dataset from pyCGM2.Report.normativeDatabaseProcedure module

        """
        pass

    def __setLayer(self):
        self.fig = plt.figure(figsize=(8.27,11.69), dpi=100,facecolor="white")
        title=u"""Mouvement  Analysis Profile \n """
        self.fig.suptitle(title)
        plt.subplots_adjust(left=None, bottom=0.5, right=None, top=None, wspace=0.5, hspace=0.5)

        ax0 = plt.subplot(1,1,1)


    def __setData(self):

        suffixPlus = "_" + self.m_pointLabelSuffix if self.m_pointLabelSuffix is not None else ""
        N = 9
        overall = (self.m_analysis.gvs["LPelvisAngles"+suffixPlus,"Left"]["mean"][0],
                     0,
                     0,
                     self.m_analysis.gvs["LPelvisAngles"+suffixPlus,"Left"]["mean"][1],
                     0,
                     self.m_analysis.gvs["LPelvisAngles"+suffixPlus,"Left"]["mean"][2],
                     0,
                     0,
                      self.m_analysis.gps["Overall"]["mean"][0])
        left = (0,
                 self.m_analysis.gvs["LHipAngles"+suffixPlus,"Left"]["mean"][0],
                 self.m_analysis.gvs["LKneeAngles"+suffixPlus,"Left"]["mean"][0],
                  0,
                  self.m_analysis.gvs["LHipAngles"+suffixPlus,"Left"]["mean"][1],
                  0,
                  self.m_analysis.gvs["LHipAngles"+suffixPlus,"Left"]["mean"][2],
                  self.m_analysis.gvs["LFootProgressAngles"+suffixPlus,"Left"]["mean"][2],
                  self.m_analysis.gps["Context"]["Left"]["mean"][0])

        right = (0,
                 self.m_analysis.gvs["RHipAngles"+suffixPlus,"Right"]["mean"][0],
                 self.m_analysis.gvs["RKneeAngles"+suffixPlus,"Right"]["mean"][0],
                  0,
                  self.m_analysis.gvs["RHipAngles"+suffixPlus,"Right"]["mean"][1],
                  0,
                  self.m_analysis.gvs["RHipAngles"+suffixPlus,"Right"]["mean"][2],
                  self.m_analysis.gvs["RFootProgressAngles"+suffixPlus,"Right"]["mean"][2],
                  self.m_analysis.gps["Context"]["Right"]["mean"][0])

        width = 0.35
        y_pos = [0,2,4,6,8,10,12,14,16]
        y_pos1 = [i+0.35 for i in y_pos]
        y_pos2 =  [i+0.7 for i in y_pos]
        self.fig.axes[0].bar(y_pos, overall, width, label='Overall',color='purple')
        self.fig.axes[0].bar(y_pos1 , left, width,  label='Left', color='red')
        self.fig.axes[0].bar(y_pos2 , right, width,  label='Right',color='blue')

        self.fig.axes[0].set_ylabel('Scores (deg)')

        tickLabels = [i+width / 3.0 for i in y_pos]
        self.fig.axes[0].set_xticks(tickLabels)
        self.fig.axes[0].set_xticklabels(('Pelvis Ant/ret', 'Hip Flex',  'Knee Flex',
                                       'Pelvis Up/Down', 'Hip Abd',
                                       'Pelvis Rot', 'Hip Rot',"Foot Prog",
                                       "GPS"), rotation=45)
        self.fig.axes[0].legend(loc='best')



    def plotPanel(self):
        self.__setLayer()
        self.__setData()

        return self.fig

class NormalizedKinematicsPlotViewer(AbstractPlotViewer):
    """

    """

    def __init__(self,iAnalysis,pointLabelSuffix=None,bodyPart=enums.BodyPartPlot.LowerLimb):

        """
            :Parameters:
                 - `iAnalysis` (pyCGM2.Processing.analysis.Analysis ) - Analysis instance built from AnalysisFilter
                 - `pointLabelSuffix` (str) - suffix ending conventional kinetic CGM labels
    """


        super(NormalizedKinematicsPlotViewer, self).__init__(iAnalysis)

        self.m_analysis = self.m_input
        if isinstance(self.m_analysis,pyCGM2.Processing.analysis.Analysis):
            pass
        else:
            logging.error( "[pyCGM2] error input object type. must be a pyCGM2.Core.Processing.analysis.Analysis")

        self.m_bodyPart = bodyPart

        self.m_pointLabelSuffix = pointLabelSuffix
        self.m_normativeData = None
        self.m_flagConsistencyOnly = False
        self.m_concretePlotFunction = None




    def setConcretePlotFunction(self, concreteplotFunction):
        self.m_concretePlotFunction = concreteplotFunction


    def __setLayer(self):

        self.fig = plt.figure(figsize=(8.27,11.69), dpi=100,facecolor="white")

        if self.m_concretePlotFunction.func_name in ["descriptivePlot","gaitDescriptivePlot"]:
            title=u""" Descriptive Time-normalized Kinematics \n """
        elif self.m_concretePlotFunction.func_name in ["consistencyPlot","gaitConsistencyPlot"]:
            title=u""" Consistency Time-normalized Kinematics \n """
        else :
            title=u"""\n"""

        self.fig.suptitle(title)
        plt.subplots_adjust(left=None, bottom=None, right=None, top=None, wspace=0.5, hspace=0.5)

        ax0 = plt.subplot(6,3,1)
        ax1 = plt.subplot(6,3,2)
        ax2 = plt.subplot(6,3,3)
        ax3 = plt.subplot(6,3,4)
        ax4 = plt.subplot(6,3,5)
        ax5 = plt.subplot(6,3,6)
        ax6 = plt.subplot(6,3,7)
        ax7 = plt.subplot(6,3,8)
        ax8 = plt.subplot(6,3,9)
        ax9 = plt.subplot(6,3,10)
        ax10 = plt.subplot(6,3,11)
        ax11 = plt.subplot(6,3,12)
        ax12 = plt.subplot(6,3,13)
        ax13 = plt.subplot(6,3,14)
        ax14 = plt.subplot(6,3,15)

        for ax in self.fig.axes:
            ax.set_ylabel("angle (deg)",size=8)
            ax.tick_params(axis='x', which='major', labelsize=6)
            ax.tick_params(axis='y', which='major', labelsize=6)

        ax12.set_xlabel("Cycle %",size=8)
        ax13.set_xlabel("Cycle %",size=8)
        ax14.set_xlabel("Cycle %",size=8)


        if self.m_bodyPart == enums.BodyPartPlot.LowerLimb:

            ax0.set_title("Pelvis Tilt" ,size=8)
            ax1.set_title("Pelvis Obliquity" ,size=8)
            ax2.set_title("Pelvis Rotation" ,size=8)
            ax3.set_title("Hip Flexion" ,size=8)
            ax4.set_title("Hip Adduction" ,size=8)
            ax5.set_title("Hip Rotation" ,size=8)
            ax6.set_title("Knee Flexion" ,size=8)
            ax7.set_title("Knee Adduction" ,size=8)
            ax8.set_title("Knee Rotation" ,size=8)
            ax9.set_title("Ankle dorsiflexion" ,size=8)
            ax10.set_title("Ankle supination" ,size=8)
            ax11.set_title("Foot Progression " ,size=8)
            ax12.set_title("ForeFoot dorsiflexion " ,size=8)
            ax13.set_title("ForeFoot eversion " ,size=8)
            ax14.set_title("ForeFoot Adduction " ,size=8)

            if not self.m_automaticYlim_flag:
                ax0.set_ylim([0,60])
                ax1.set_ylim([-30,30])
                ax2.set_ylim([-30,30])

                ax3.set_ylim( [-20,70])
                ax4.set_ylim([-30,30])
                ax5.set_ylim([-30,30])

                ax6.set_ylim([-15,75])
                ax7.set_ylim([-30,30])
                ax8.set_ylim([-30,30])

                ax9.set_ylim([-30,30])
                ax10.set_ylim([-30,30])
                ax11.set_ylim([-30,30])

                ax12.set_ylim([-50,30])
                ax13.set_ylim([-30,30])
                ax14.set_ylim([-30,30])

        elif self.m_bodyPart == enums.BodyPartPlot.Trunk:

            ax0.set_title("Pelvis Tilt" ,size=8)
            ax1.set_title("Pelvis Obliquity" ,size=8)
            ax2.set_title("Pelvis Rotation" ,size=8)
            ax3.set_title("Spine Flexion" ,size=8)
            ax4.set_title("Spine Adduction" ,size=8)
            ax5.set_title("Spine Rotation" ,size=8)
            ax6.set_title("Thorax Anteversion" ,size=8)
            ax7.set_title("Thorax Obliquity" ,size=8)
            ax8.set_title("Thorax Rotation" ,size=8)

            ax9.set_title("Neck Forward Tilt" ,size=8)
            ax10.set_title("Neck Lateral Tilt" ,size=8)
            ax11.set_title("Neck Rotation" ,size=8)

            ax12.set_title("Head Forward Tilt" ,size=8)
            ax13.set_title("Head Lateral Tilt" ,size=8)
            ax14.set_title("Head Rotation" ,size=8)

            if not self.m_automaticYlim_flag:
                ax0.set_ylim([0,60])
                ax1.set_ylim([-30,30])
                ax2.set_ylim([-30,30])

                ax3.set_ylim( [-30,30])
                ax4.set_ylim([-30,30])
                ax5.set_ylim([-30,30])

                ax6.set_ylim([-30,30])
                ax7.set_ylim([-30,30])
                ax8.set_ylim([-30,30])

                ax9.set_ylim([-30,30])
                ax10.set_ylim([-30,30])
                ax11.set_ylim([-30,30])

                ax12.set_ylim([-50,30])
                ax13.set_ylim([-30,30])
                ax14.set_ylim([-30,30])

        elif self.m_bodyPart == enums.BodyPartPlot.UpperLimb:

            ax0.set_title("Shoulder flexion" ,size=8)
            ax1.set_title("Shoulder Adduction" ,size=8)
            ax2.set_title("Shoulder Rotation" ,size=8)
            ax3.set_title("Elbow Flexion" ,size=8)
            # ax4.set_title("Elbow Adduction" ,size=8)
            # ax5.set_title("Spine Rotation" ,size=8)
            ax6.set_title("Ulnar Deviation" ,size=8)
            ax7.set_title("Wrist Extension" ,size=8)
            ax8.set_title("Wrist Rotation" ,size=8)

            if not self.m_automaticYlim_flag:
                ax0.set_ylim([-60,60])
                ax1.set_ylim([-30,30])
                ax2.set_ylim([-30,30])

                ax3.set_ylim( [-60,60])
                ax4.set_ylim([-30,30])
                ax5.set_ylim([-30,30])

                ax6.set_ylim([-30,30])
                ax7.set_ylim([-30,30])
                ax8.set_ylim([0,160])

                ax9.set_ylim([-30,30])
                ax10.set_ylim([-30,30])
                ax11.set_ylim([-30,30])

                ax12.set_ylim([-50,30])
                ax13.set_ylim([-30,30])
                ax14.set_ylim([-30,30])

    def setNormativeDataset(self,iNormativeDataSet):
        """
            **Description :** set a normative gait dataset

            :Parameters:
                 - `iNormativeDataSet` (a class of the pyCGM2.Report.normativeDataset module) - normative gait dataset from pyCGM2.Report.normativeDatabaseProcedure module

        """
        # iNormativeDataSet.constructNormativeData()
        self.m_normativeData = iNormativeDataSet.data

    def __setData(self):
        suffixPlus = "_" + self.m_pointLabelSuffix if self.m_pointLabelSuffix is not None else ""

        if self.m_bodyPart == enums.BodyPartPlot.LowerLimb:

            # pelvis
            self.m_concretePlotFunction(self.fig.axes[0],self.m_analysis.kinematicStats,
                    "LPelvisAngles"+suffixPlus,"Left",0, color="red",customLimits=None)
            self.m_concretePlotFunction(self.fig.axes[0],self.m_analysis.kinematicStats,
                    "RPelvisAngles"+suffixPlus,"Right",0, color="blue",customLimits=None)

            self.m_concretePlotFunction(self.fig.axes[1],self.m_analysis.kinematicStats,
                    "LPelvisAngles"+suffixPlus,"Left",1, color="red",customLimits=None)
            self.m_concretePlotFunction(self.fig.axes[1],self.m_analysis.kinematicStats,
                    "RPelvisAngles"+suffixPlus,"Right",1, color="blue",customLimits=None)

            self.m_concretePlotFunction(self.fig.axes[2],self.m_analysis.kinematicStats,
                    "LPelvisAngles"+suffixPlus,"Left",2, color="red",customLimits=None)
            self.m_concretePlotFunction(self.fig.axes[2],self.m_analysis.kinematicStats,
                    "RPelvisAngles"+suffixPlus,"Right",2, color="blue",customLimits=None)

            # hip
            self.m_concretePlotFunction(self.fig.axes[3],self.m_analysis.kinematicStats,
                    "LHipAngles"+suffixPlus,"Left",0, color="red",customLimits=None)
            self.m_concretePlotFunction(self.fig.axes[3],self.m_analysis.kinematicStats,
                    "RHipAngles"+suffixPlus,"Right",0, color="blue",customLimits=None)

            self.m_concretePlotFunction(self.fig.axes[4],self.m_analysis.kinematicStats,
                    "LHipAngles"+suffixPlus,"Left",1, color="red",customLimits=None)
            self.m_concretePlotFunction(self.fig.axes[4],self.m_analysis.kinematicStats,
                    "RHipAngles"+suffixPlus,"Right",1, color="blue",customLimits=None)

            self.m_concretePlotFunction(self.fig.axes[5],self.m_analysis.kinematicStats,
                    "LHipAngles"+suffixPlus,"Left",2, color="red",customLimits=None)
            self.m_concretePlotFunction(self.fig.axes[5],self.m_analysis.kinematicStats,
                    "RHipAngles"+suffixPlus,"Right",2, color="blue",customLimits=None)

            # knee
            self.m_concretePlotFunction(self.fig.axes[6],self.m_analysis.kinematicStats,
                    "LKneeAngles"+suffixPlus,"Left",0, color="red",customLimits=None)
            self.m_concretePlotFunction(self.fig.axes[6],self.m_analysis.kinematicStats,
                    "RKneeAngles"+suffixPlus,"Right",0, color="blue",customLimits=None)

            self.m_concretePlotFunction(self.fig.axes[7],self.m_analysis.kinematicStats,
                    "LKneeAngles"+suffixPlus,"Left",1, color="red",customLimits=None)
            self.m_concretePlotFunction(self.fig.axes[7],self.m_analysis.kinematicStats,
                    "RKneeAngles"+suffixPlus,"Right",1, color="blue",customLimits=None)

            self.m_concretePlotFunction(self.fig.axes[8],self.m_analysis.kinematicStats,
                    "LKneeAngles"+suffixPlus,"Left",2, color="red",customLimits=None)
            self.m_concretePlotFunction(self.fig.axes[8],self.m_analysis.kinematicStats,
                    "RKneeAngles"+suffixPlus,"Right",2, color="blue",customLimits=None)

            # ankle
            self.m_concretePlotFunction(self.fig.axes[9],self.m_analysis.kinematicStats,
                    "LAnkleAngles"+suffixPlus,"Left",0, color="red",customLimits=None)
            self.m_concretePlotFunction(self.fig.axes[9],self.m_analysis.kinematicStats,
                    "RAnkleAngles"+suffixPlus,"Right",0, color="blue",customLimits=None)

            self.m_concretePlotFunction(self.fig.axes[10],self.m_analysis.kinematicStats,
                    "LAnkleAngles"+suffixPlus,"Left",1, color="red",customLimits=None)
            self.m_concretePlotFunction(self.fig.axes[10],self.m_analysis.kinematicStats,
                    "RAnkleAngles"+suffixPlus,"Right",1, color="blue",customLimits=None)

            # foot progress
            self.m_concretePlotFunction(self.fig.axes[11],self.m_analysis.kinematicStats,
                    "LFootProgressAngles"+suffixPlus,"Left",2, color="red",customLimits=None)
            self.m_concretePlotFunction(self.fig.axes[11],self.m_analysis.kinematicStats,
                    "RFootProgressAngles"+suffixPlus,"Right",2, color="blue",customLimits=None)

            # ForeFoot
            self.m_concretePlotFunction(self.fig.axes[12],self.m_analysis.kinematicStats,
                    "LForeFootAngles"+suffixPlus,"Left",0, color="red",customLimits=None)
            self.m_concretePlotFunction(self.fig.axes[12],self.m_analysis.kinematicStats,
                    "RForeFootAngles"+suffixPlus,"Right",0, color="blue",customLimits=None)

            self.m_concretePlotFunction(self.fig.axes[13],self.m_analysis.kinematicStats,
                    "LForeFootAngles"+suffixPlus,"Left",1, color="red",customLimits=None)
            self.m_concretePlotFunction(self.fig.axes[13],self.m_analysis.kinematicStats,
                    "RForeFootAngles"+suffixPlus,"Right",1, color="blue",customLimits=None)

            self.m_concretePlotFunction(self.fig.axes[14],self.m_analysis.kinematicStats,
                    "LForeFootAngles"+suffixPlus,"Left",2, color="red",customLimits=None)
            self.m_concretePlotFunction(self.fig.axes[14],self.m_analysis.kinematicStats,
                    "RForeFootAngles"+suffixPlus,"Right",2, color="blue",customLimits=None)

        elif self.m_bodyPart == enums.BodyPartPlot.Trunk:
            # pelvis
            self.m_concretePlotFunction(self.fig.axes[0],self.m_analysis.kinematicStats,
                    "LPelvisAngles"+suffixPlus,"Left",0, color="red",customLimits=None)
            self.m_concretePlotFunction(self.fig.axes[0],self.m_analysis.kinematicStats,
                    "RPelvisAngles"+suffixPlus,"Right",0, color="blue",customLimits=None)

            self.m_concretePlotFunction(self.fig.axes[1],self.m_analysis.kinematicStats,
                    "LPelvisAngles"+suffixPlus,"Left",1, color="red",customLimits=None)
            self.m_concretePlotFunction(self.fig.axes[1],self.m_analysis.kinematicStats,
                    "RPelvisAngles"+suffixPlus,"Right",1, color="blue",customLimits=None)

            self.m_concretePlotFunction(self.fig.axes[2],self.m_analysis.kinematicStats,
                    "LPelvisAngles"+suffixPlus,"Left",2, color="red",customLimits=None)
            self.m_concretePlotFunction(self.fig.axes[2],self.m_analysis.kinematicStats,
                    "RPelvisAngles"+suffixPlus,"Right",2, color="blue",customLimits=None)

            # Spine
            self.m_concretePlotFunction(self.fig.axes[3],self.m_analysis.kinematicStats,
                    "LSpineAngles"+suffixPlus,"Left",0, color="red",customLimits=None)
            self.m_concretePlotFunction(self.fig.axes[3],self.m_analysis.kinematicStats,
                    "RSpineAngles"+suffixPlus,"Right",0, color="blue",customLimits=None)

            self.m_concretePlotFunction(self.fig.axes[4],self.m_analysis.kinematicStats,
                    "LSpineAngles"+suffixPlus,"Left",1, color="red",customLimits=None)
            self.m_concretePlotFunction(self.fig.axes[4],self.m_analysis.kinematicStats,
                    "RSpineAngles"+suffixPlus,"Right",1, color="blue",customLimits=None)

            self.m_concretePlotFunction(self.fig.axes[5],self.m_analysis.kinematicStats,
                    "LSpineAngles"+suffixPlus,"Left",2, color="red",customLimits=None)
            self.m_concretePlotFunction(self.fig.axes[5],self.m_analysis.kinematicStats,
                    "RSpineAngles"+suffixPlus,"Right",2, color="blue",customLimits=None)

            # Thorax
            self.m_concretePlotFunction(self.fig.axes[6],self.m_analysis.kinematicStats,
                    "LThoraxAngles"+suffixPlus,"Left",0, color="red",customLimits=None)
            self.m_concretePlotFunction(self.fig.axes[6],self.m_analysis.kinematicStats,
                    "RThoraxAngles"+suffixPlus,"Right",0, color="blue",customLimits=None)

            self.m_concretePlotFunction(self.fig.axes[7],self.m_analysis.kinematicStats,
                    "LThoraxAngles"+suffixPlus,"Left",1, color="red",customLimits=None)
            self.m_concretePlotFunction(self.fig.axes[7],self.m_analysis.kinematicStats,
                    "RThoraxAngles"+suffixPlus,"Right",1, color="blue",customLimits=None)

            self.m_concretePlotFunction(self.fig.axes[8],self.m_analysis.kinematicStats,
                    "LThoraxAngles"+suffixPlus,"Left",2, color="red",customLimits=None)
            self.m_concretePlotFunction(self.fig.axes[8],self.m_analysis.kinematicStats,
                    "RThoraxAngles"+suffixPlus,"Right",2, color="blue",customLimits=None)


            # Neck
            self.m_concretePlotFunction(self.fig.axes[9],self.m_analysis.kinematicStats,
                    "LNeckAngles"+suffixPlus,"Left",0, color="red",customLimits=None)
            self.m_concretePlotFunction(self.fig.axes[9],self.m_analysis.kinematicStats,
                    "RNeckAngles"+suffixPlus,"Right",0, color="blue",customLimits=None)

            self.m_concretePlotFunction(self.fig.axes[10],self.m_analysis.kinematicStats,
                    "LNeckAngles"+suffixPlus,"Left",1, color="red",customLimits=None)
            self.m_concretePlotFunction(self.fig.axes[10],self.m_analysis.kinematicStats,
                    "RNeckAngles"+suffixPlus,"Right",1, color="blue",customLimits=None)

            self.m_concretePlotFunction(self.fig.axes[11],self.m_analysis.kinematicStats,
                    "LNeckAngles"+suffixPlus,"Left",2, color="red",customLimits=None)
            self.m_concretePlotFunction(self.fig.axes[11],self.m_analysis.kinematicStats,
                    "RNeckAngles"+suffixPlus,"Right",2, color="blue",customLimits=None)


            # Head
            self.m_concretePlotFunction(self.fig.axes[12],self.m_analysis.kinematicStats,
                    "LHeadAngles"+suffixPlus,"Left",0, color="red",customLimits=None)
            self.m_concretePlotFunction(self.fig.axes[12],self.m_analysis.kinematicStats,
                    "RHeadAngles"+suffixPlus,"Right",0, color="blue",customLimits=None)

            self.m_concretePlotFunction(self.fig.axes[13],self.m_analysis.kinematicStats,
                    "LHeadAngles"+suffixPlus,"Left",1, color="red",customLimits=None)
            self.m_concretePlotFunction(self.fig.axes[13],self.m_analysis.kinematicStats,
                    "RHeadAngles"+suffixPlus,"Right",1, color="blue",customLimits=None)

            self.m_concretePlotFunction(self.fig.axes[14],self.m_analysis.kinematicStats,
                    "LHeadAngles"+suffixPlus,"Left",2, color="red",customLimits=None)
            self.m_concretePlotFunction(self.fig.axes[14],self.m_analysis.kinematicStats,
                    "RHeadAngles"+suffixPlus,"Right",2, color="blue",customLimits=None)



        elif self.m_bodyPart == enums.BodyPartPlot.UpperLimb:
            # shoulder
            self.m_concretePlotFunction(self.fig.axes[0],self.m_analysis.kinematicStats,
                    "LShoulderAngles"+suffixPlus,"Left",0, color="red",customLimits=None)
            self.m_concretePlotFunction(self.fig.axes[0],self.m_analysis.kinematicStats,
                    "RShoulderAngles"+suffixPlus,"Right",0, color="blue",customLimits=None)

            self.m_concretePlotFunction(self.fig.axes[1],self.m_analysis.kinematicStats,
                    "LShoulderAngles"+suffixPlus,"Left",1, color="red",customLimits=None)
            self.m_concretePlotFunction(self.fig.axes[1],self.m_analysis.kinematicStats,
                    "RShoulderAngles"+suffixPlus,"Right",1, color="blue",customLimits=None)

            self.m_concretePlotFunction(self.fig.axes[2],self.m_analysis.kinematicStats,
                    "LShoulderAngles"+suffixPlus,"Left",2, color="red",customLimits=None)
            self.m_concretePlotFunction(self.fig.axes[2],self.m_analysis.kinematicStats,
                    "RShoulderAngles"+suffixPlus,"Right",2, color="blue",customLimits=None)

            # elbow
            self.m_concretePlotFunction(self.fig.axes[3],self.m_analysis.kinematicStats,
                    "LElbowAngles"+suffixPlus,"Left",0, color="red",customLimits=None)
            self.m_concretePlotFunction(self.fig.axes[3],self.m_analysis.kinematicStats,
                    "RElbowAngles"+suffixPlus,"Right",0, color="blue",customLimits=None)

            # self.m_concretePlotFunction(self.fig.axes[4],self.m_analysis.kinematicStats,
            #         "LElbowAngles"+suffixPlus,"Left",1, color="red",customLimits=None)
            # self.m_concretePlotFunction(self.fig.axes[4],self.m_analysis.kinematicStats,
            #         "LElbowAngles"+suffixPlus,"Right",1, color="blue",customLimits=None)
            #
            # self.m_concretePlotFunction(self.fig.axes[5],self.m_analysis.kinematicStats,
            #         "LElbowAngles"+suffixPlus,"Left",2, color="red",customLimits=None)
            # self.m_concretePlotFunction(self.fig.axes[5],self.m_analysis.kinematicStats,
            #         "RElbowAngles"+suffixPlus,"Right",2, color="blue",customLimits=None)

            # wrist
            self.m_concretePlotFunction(self.fig.axes[6],self.m_analysis.kinematicStats,
                    "LWristAngles"+suffixPlus,"Left",0, color="red",customLimits=None)
            self.m_concretePlotFunction(self.fig.axes[6],self.m_analysis.kinematicStats,
                    "RWristAngles"+suffixPlus,"Right",0, color="blue",customLimits=None)

            self.m_concretePlotFunction(self.fig.axes[7],self.m_analysis.kinematicStats,
                    "LWristAngles"+suffixPlus,"Left",1, color="red",customLimits=None)
            self.m_concretePlotFunction(self.fig.axes[7],self.m_analysis.kinematicStats,
                    "RWristAngles"+suffixPlus,"Right",1, color="blue",customLimits=None)

            self.m_concretePlotFunction(self.fig.axes[8],self.m_analysis.kinematicStats,
                    "LWristAngles"+suffixPlus,"Left",2, color="red",customLimits=None)
            self.m_concretePlotFunction(self.fig.axes[8],self.m_analysis.kinematicStats,
                    "RWristAngles"+suffixPlus,"Right",2, color="blue",customLimits=None)



    def plotPanel(self):

        if self.m_concretePlotFunction is None:
            raise Exception ("[pyCGM2] need definition of the concrete plot function")

        self.__setLayer()
        self.__setData()

        if self.m_normativeData is not None:
            if self.m_bodyPart == enums.BodyPartPlot.LowerLimb:

                if "PelvisAngles" in self.m_normativeData.keys():
                    self.fig.axes[0].fill_between(np.linspace(0,100,self.m_normativeData["PelvisAngles"]["mean"].shape[0]),
                        self.m_normativeData["PelvisAngles"]["mean"][:,0]-self.m_normativeData["PelvisAngles"]["sd"][:,0],
                        self.m_normativeData["PelvisAngles"]["mean"][:,0]+self.m_normativeData["PelvisAngles"]["sd"][:,0],
                        facecolor="green", alpha=0.5,linewidth=0)

                    self.fig.axes[1].fill_between(np.linspace(0,100,self.m_normativeData["PelvisAngles"]["mean"].shape[0]),
                        self.m_normativeData["PelvisAngles"]["mean"][:,1]-self.m_normativeData["PelvisAngles"]["sd"][:,1],
                        self.m_normativeData["PelvisAngles"]["mean"][:,1]+self.m_normativeData["PelvisAngles"]["sd"][:,1],
                        facecolor="green", alpha=0.5,linewidth=0)

                    self.fig.axes[2].fill_between(np.linspace(0,100,self.m_normativeData["PelvisAngles"]["mean"].shape[0]),
                        self.m_normativeData["PelvisAngles"]["mean"][:,2]-self.m_normativeData["PelvisAngles"]["sd"][:,2],
                        self.m_normativeData["PelvisAngles"]["mean"][:,2]+self.m_normativeData["PelvisAngles"]["sd"][:,2],
                        facecolor="green", alpha=0.5,linewidth=0)

                if "HipAngles" in self.m_normativeData.keys():
                    self.fig.axes[3].fill_between(np.linspace(0,100,self.m_normativeData["HipAngles"]["mean"].shape[0]),
                        self.m_normativeData["HipAngles"]["mean"][:,0]-self.m_normativeData["HipAngles"]["sd"][:,0],
                        self.m_normativeData["HipAngles"]["mean"][:,0]+self.m_normativeData["HipAngles"]["sd"][:,0],
                        facecolor="green", alpha=0.5,linewidth=0)

                    self.fig.axes[4].fill_between(np.linspace(0,100,self.m_normativeData["HipAngles"]["mean"].shape[0]),
                        self.m_normativeData["HipAngles"]["mean"][:,1]-self.m_normativeData["HipAngles"]["sd"][:,1],
                        self.m_normativeData["HipAngles"]["mean"][:,1]+self.m_normativeData["HipAngles"]["sd"][:,1],
                        facecolor="green", alpha=0.5,linewidth=0)

                    self.fig.axes[5].fill_between(np.linspace(0,100,self.m_normativeData["HipAngles"]["mean"].shape[0]),
                        self.m_normativeData["HipAngles"]["mean"][:,2]-self.m_normativeData["HipAngles"]["sd"][:,2],
                        self.m_normativeData["HipAngles"]["mean"][:,2]+self.m_normativeData["HipAngles"]["sd"][:,2],
                        facecolor="green", alpha=0.5,linewidth=0)

                if "KneeAngles" in self.m_normativeData.keys():
                    self.fig.axes[6].fill_between(np.linspace(0,100,self.m_normativeData["KneeAngles"]["mean"].shape[0]),
                        self.m_normativeData["KneeAngles"]["mean"][:,0]-self.m_normativeData["KneeAngles"]["sd"][:,0],
                        self.m_normativeData["KneeAngles"]["mean"][:,0]+self.m_normativeData["KneeAngles"]["sd"][:,0],
                        facecolor="green", alpha=0.5,linewidth=0)

                    self.fig.axes[7].fill_between(np.linspace(0,100,self.m_normativeData["KneeAngles"]["mean"].shape[0]),
                        self.m_normativeData["KneeAngles"]["mean"][:,1]-self.m_normativeData["KneeAngles"]["sd"][:,1],
                        self.m_normativeData["KneeAngles"]["mean"][:,1]+self.m_normativeData["KneeAngles"]["sd"][:,1],
                        facecolor="green", alpha=0.5,linewidth=0)

                    self.fig.axes[8].fill_between(np.linspace(0,100,self.m_normativeData["KneeAngles"]["mean"].shape[0]),
                        self.m_normativeData["KneeAngles"]["mean"][:,2]-self.m_normativeData["KneeAngles"]["sd"][:,2],
                        self.m_normativeData["KneeAngles"]["mean"][:,2]+self.m_normativeData["KneeAngles"]["sd"][:,2],
                        facecolor="green", alpha=0.5,linewidth=0)

                if "AnkleAngles" in self.m_normativeData.keys():
                    self.fig.axes[9].fill_between(np.linspace(0,100,self.m_normativeData["AnkleAngles"]["mean"].shape[0]),
                        self.m_normativeData["AnkleAngles"]["mean"][:,0]-self.m_normativeData["AnkleAngles"]["sd"][:,0],
                        self.m_normativeData["AnkleAngles"]["mean"][:,0]+self.m_normativeData["AnkleAngles"]["sd"][:,0],
                        facecolor="green", alpha=0.5,linewidth=0)

                    self.fig.axes[10].fill_between(np.linspace(0,100,self.m_normativeData["AnkleAngles"]["mean"].shape[0]),
                        self.m_normativeData["AnkleAngles"]["mean"][:,1]-self.m_normativeData["AnkleAngles"]["sd"][:,1],
                        self.m_normativeData["AnkleAngles"]["mean"][:,1]+self.m_normativeData["AnkleAngles"]["sd"][:,1],
                        facecolor="green", alpha=0.5,linewidth=0)

                if "FootProgressAngles" in self.m_normativeData.keys():
                    self.fig.axes[11].fill_between(np.linspace(0,100,self.m_normativeData["FootProgressAngles"]["mean"].shape[0]),
                        self.m_normativeData["FootProgressAngles"]["mean"][:,2]-self.m_normativeData["FootProgressAngles"]["sd"][:,2],
                        self.m_normativeData["FootProgressAngles"]["mean"][:,2]+self.m_normativeData["FootProgressAngles"]["sd"][:,2],
                        facecolor="green", alpha=0.5,linewidth=0)

            elif self.m_bodyPart == enums.BodyPartPlot.Trunk:


                if "PelvisAngles" in self.m_normativeData.keys():
                    self.fig.axes[0].fill_between(np.linspace(0,100,self.m_normativeData["PelvisAngles"]["mean"].shape[0]),
                        self.m_normativeData["PelvisAngles"]["mean"][:,0]-self.m_normativeData["PelvisAngles"]["sd"][:,0],
                        self.m_normativeData["PelvisAngles"]["mean"][:,0]+self.m_normativeData["PelvisAngles"]["sd"][:,0],
                        facecolor="green", alpha=0.5,linewidth=0)

                    self.fig.axes[1].fill_between(np.linspace(0,100,self.m_normativeData["PelvisAngles"]["mean"].shape[0]),
                        self.m_normativeData["PelvisAngles"]["mean"][:,1]-self.m_normativeData["PelvisAngles"]["sd"][:,1],
                        self.m_normativeData["PelvisAngles"]["mean"][:,1]+self.m_normativeData["PelvisAngles"]["sd"][:,1],
                        facecolor="green", alpha=0.5,linewidth=0)

                    self.fig.axes[2].fill_between(np.linspace(0,100,self.m_normativeData["PelvisAngles"]["mean"].shape[0]),
                        self.m_normativeData["PelvisAngles"]["mean"][:,2]-self.m_normativeData["PelvisAngles"]["sd"][:,2],
                        self.m_normativeData["PelvisAngles"]["mean"][:,2]+self.m_normativeData["PelvisAngles"]["sd"][:,2],
                        facecolor="green", alpha=0.5,linewidth=0)

                if "SpineAngles" in self.m_normativeData.keys():
                    self.fig.axes[3].fill_between(np.linspace(0,100,self.m_normativeData["SpineAngles"]["mean"].shape[0]),
                        self.m_normativeData["SpineAngles"]["mean"][:,0]-self.m_normativeData["SpineAngles"]["sd"][:,0],
                        self.m_normativeData["SpineAngles"]["mean"][:,0]+self.m_normativeData["SpineAngles"]["sd"][:,0],
                        facecolor="green", alpha=0.5,linewidth=0)

                    self.fig.axes[4].fill_between(np.linspace(0,100,self.m_normativeData["SpineAngles"]["mean"].shape[0]),
                        self.m_normativeData["SpineAngles"]["mean"][:,1]-self.m_normativeData["SpineAngles"]["sd"][:,1],
                        self.m_normativeData["SpineAngles"]["mean"][:,1]+self.m_normativeData["SpineAngles"]["sd"][:,1],
                        facecolor="green", alpha=0.5,linewidth=0)

                    self.fig.axes[5].fill_between(np.linspace(0,100,self.m_normativeData["SpineAngles"]["mean"].shape[0]),
                        self.m_normativeData["SpineAngles"]["mean"][:,2]-self.m_normativeData["SpineAngles"]["sd"][:,2],
                        self.m_normativeData["SpineAngles"]["mean"][:,2]+self.m_normativeData["SpineAngles"]["sd"][:,2],
                        facecolor="green", alpha=0.5,linewidth=0)

                if "ThoraxAngles" in self.m_normativeData.keys():
                    self.fig.axes[6].fill_between(np.linspace(0,100,self.m_normativeData["ThoraxAngles"]["mean"].shape[0]),
                        self.m_normativeData["ThoraxAngles"]["mean"][:,0]-self.m_normativeData["ThoraxAngles"]["sd"][:,0],
                        self.m_normativeData["ThoraxAngles"]["mean"][:,0]+self.m_normativeData["ThoraxAngles"]["sd"][:,0],
                        facecolor="green", alpha=0.5,linewidth=0)

                    self.fig.axes[7].fill_between(np.linspace(0,100,self.m_normativeData["ThoraxAngles"]["mean"].shape[0]),
                        self.m_normativeData["ThoraxAngles"]["mean"][:,1]-self.m_normativeData["ThoraxAngles"]["sd"][:,1],
                        self.m_normativeData["ThoraxAngles"]["mean"][:,1]+self.m_normativeData["ThoraxAngles"]["sd"][:,1],
                        facecolor="green", alpha=0.5,linewidth=0)

                    self.fig.axes[8].fill_between(np.linspace(0,100,self.m_normativeData["ThoraxAngles"]["mean"].shape[0]),
                        self.m_normativeData["ThoraxAngles"]["mean"][:,2]-self.m_normativeData["ThoraxAngles"]["sd"][:,2],
                        self.m_normativeData["ThoraxAngles"]["mean"][:,2]+self.m_normativeData["ThoraxAngles"]["sd"][:,2],
                        facecolor="green", alpha=0.5,linewidth=0)

                if "NeckAngles" in self.m_normativeData.keys():
                    self.fig.axes[9].fill_between(np.linspace(0,100,self.m_normativeData["NeckAngles"]["mean"].shape[0]),
                        self.m_normativeData["NeckAngles"]["mean"][:,0]-self.m_normativeData["NeckAngles"]["sd"][:,0],
                        self.m_normativeData["NeckAngles"]["mean"][:,0]+self.m_normativeData["NeckAngles"]["sd"][:,0],
                        facecolor="green", alpha=0.5,linewidth=0)

                    self.fig.axes[10].fill_between(np.linspace(0,100,self.m_normativeData["NeckAngles"]["mean"].shape[0]),
                        self.m_normativeData["NeckAngles"]["mean"][:,1]-self.m_normativeData["NeckAngles"]["sd"][:,1],
                        self.m_normativeData["NeckAngles"]["mean"][:,1]+self.m_normativeData["NeckAngles"]["sd"][:,1],
                        facecolor="green", alpha=0.5,linewidth=0)

                    self.fig.axes[11].fill_between(np.linspace(0,100,self.m_normativeData["NeckAngles"]["mean"].shape[0]),
                        self.m_normativeData["NeckAngles"]["mean"][:,2]-self.m_normativeData["NeckAngles"]["sd"][:,2],
                        self.m_normativeData["NeckAngles"]["mean"][:,2]+self.m_normativeData["NeckAngles"]["sd"][:,2],
                        facecolor="green", alpha=0.5,linewidth=0)

                if "HeadAngles" in self.m_normativeData.keys():
                    self.fig.axes[12].fill_between(np.linspace(0,100,self.m_normativeData["HeadAngles"]["mean"].shape[0]),
                        self.m_normativeData["HeadAngles"]["mean"][:,0]-self.m_normativeData["HeadAngles"]["sd"][:,0],
                        self.m_normativeData["HeadAngles"]["mean"][:,0]+self.m_normativeData["HeadAngles"]["sd"][:,0],
                        facecolor="green", alpha=0.5,linewidth=0)

                    self.fig.axes[13].fill_between(np.linspace(0,100,self.m_normativeData["HeadAngles"]["mean"].shape[0]),
                        self.m_normativeData["HeadAngles"]["mean"][:,1]-self.m_normativeData["HeadAngles"]["sd"][:,1],
                        self.m_normativeData["HeadAngles"]["mean"][:,1]+self.m_normativeData["HeadAngles"]["sd"][:,1],
                        facecolor="green", alpha=0.5,linewidth=0)

                    self.fig.axes[14].fill_between(np.linspace(0,100,self.m_normativeData["HeadAngles"]["mean"].shape[0]),
                        self.m_normativeData["HeadAngles"]["mean"][:,2]-self.m_normativeData["HeadAngles"]["sd"][:,2],
                        self.m_normativeData["HeadAngles"]["mean"][:,2]+self.m_normativeData["HeadAngles"]["sd"][:,2],
                        facecolor="green", alpha=0.5,linewidth=0)

            elif self.m_bodyPart == enums.BodyPartPlot.UpperLimb:


                if "ShoulderAngles" in self.m_normativeData.keys():
                    self.fig.axes[0].fill_between(np.linspace(0,100,self.m_normativeData["ShoulderAngles"]["mean"].shape[0]),
                        self.m_normativeData["ShoulderAngles"]["mean"][:,0]-self.m_normativeData["ShoulderAngles"]["sd"][:,0],
                        self.m_normativeData["ShoulderAngles"]["mean"][:,0]+self.m_normativeData["ShoulderAngles"]["sd"][:,0],
                        facecolor="green", alpha=0.5,linewidth=0)

                    self.fig.axes[1].fill_between(np.linspace(0,100,self.m_normativeData["ShoulderAngles"]["mean"].shape[0]),
                        self.m_normativeData["ShoulderAngles"]["mean"][:,1]-self.m_normativeData["ShoulderAngles"]["sd"][:,1],
                        self.m_normativeData["ShoulderAngles"]["mean"][:,1]+self.m_normativeData["ShoulderAngles"]["sd"][:,1],
                        facecolor="green", alpha=0.5,linewidth=0)

                    self.fig.axes[2].fill_between(np.linspace(0,100,self.m_normativeData["ShoulderAngles"]["mean"].shape[0]),
                        self.m_normativeData["ShoulderAngles"]["mean"][:,2]-self.m_normativeData["ShoulderAngles"]["sd"][:,2],
                        self.m_normativeData["ShoulderAngles"]["mean"][:,2]+self.m_normativeData["ShoulderAngles"]["sd"][:,2],
                        facecolor="green", alpha=0.5,linewidth=0)

                if "ElbowAngles" in self.m_normativeData.keys():
                    self.fig.axes[3].fill_between(np.linspace(0,100,self.m_normativeData["ElbowAngles"]["mean"].shape[0]),
                        self.m_normativeData["ElbowAngles"]["mean"][:,0]-self.m_normativeData["ElbowAngles"]["sd"][:,0],
                        self.m_normativeData["ElbowAngles"]["mean"][:,0]+self.m_normativeData["ElbowAngles"]["sd"][:,0],
                        facecolor="green", alpha=0.5,linewidth=0)

                    # self.fig.axes[4].fill_between(np.linspace(0,100,self.m_normativeData["ElbowAngles"]["mean"].shape[0]),
                    #     self.m_normativeData["ElbowAngles"]["mean"][:,1]-self.m_normativeData["ElbowAngles"]["sd"][:,1],
                    #     self.m_normativeData["ElbowAngles"]["mean"][:,1]+self.m_normativeData["ElbowAngles"]["sd"][:,1],
                    #     facecolor="green", alpha=0.5,linewidth=0)
                    #
                    # self.fig.axes[5].fill_between(np.linspace(0,100,self.m_normativeData["ElbowAngles"]["mean"].shape[0]),
                    #     self.m_normativeData["ElbowAngles"]["mean"][:,2]-self.m_normativeData["ElbowAngles"]["sd"][:,2],
                    #     self.m_normativeData["ElbowAngles"]["mean"][:,2]+self.m_normativeData["ElbowAngles"]["sd"][:,2],
                    #     facecolor="green", alpha=0.5,linewidth=0)


                if "WristAngles" in self.m_normativeData.keys():
                    self.fig.axes[6].fill_between(np.linspace(0,100,self.m_normativeData["WristAngles"]["mean"].shape[0]),
                        self.m_normativeData["WristAngles"]["mean"][:,0]-self.m_normativeData["WristAngles"]["sd"][:,0],
                        self.m_normativeData["WristAngles"]["mean"][:,0]+self.m_normativeData["WristAngles"]["sd"][:,0],
                        facecolor="green", alpha=0.5,linewidth=0)

                    self.fig.axes[7].fill_between(np.linspace(0,100,self.m_normativeData["WristAngles"]["mean"].shape[0]),
                        self.m_normativeData["WristAngles"]["mean"][:,1]-self.m_normativeData["WristAngles"]["sd"][:,1],
                        self.m_normativeData["WristAngles"]["mean"][:,1]+self.m_normativeData["WristAngles"]["sd"][:,1],
                        facecolor="green", alpha=0.5,linewidth=0)

                    self.fig.axes[8].fill_between(np.linspace(0,100,self.m_normativeData["WristAngles"]["mean"].shape[0]),
                        self.m_normativeData["WristAngles"]["mean"][:,2]-self.m_normativeData["WristAngles"]["sd"][:,2],
                        self.m_normativeData["WristAngles"]["mean"][:,2]+self.m_normativeData["WristAngles"]["sd"][:,2],
                        facecolor="green", alpha=0.5,linewidth=0)

        return self.fig

class TemporalKinematicsPlotViewer(AbstractPlotViewer):
    """

    """

    def __init__(self,iAcq,pointLabelSuffix=None,bodyPart=enums.BodyPartPlot.LowerLimb):

        """
            :Parameters:
                 - `pointLabelSuffix` (str) - suffix ending conventional kinetic CGM labels
        """


        super(TemporalKinematicsPlotViewer, self).__init__(iAcq)

        self.m_acq = self.m_input
        if isinstance(self.m_input,pyCGM2.btk.btkAcquisition):
            pass
        else:
            logging.error( "[pyCGM2] error input object type. must be a ma.Trial")


        self.m_bodyPart = bodyPart
        self.m_pointLabelSuffix = pointLabelSuffix
        self.m_normativeData = None


    def __setLayer(self):

        self.fig = plt.figure(figsize=(8.27,11.69), dpi=100,facecolor="white")
        title=u""" Time Kinematics \n """
        self.fig.suptitle(title)
        plt.subplots_adjust(left=None, bottom=None, right=None, top=None, wspace=0.5, hspace=0.5)

        ax0 = plt.subplot(6,3,1)# Pelvis X
        ax1 = plt.subplot(6,3,2)# Pelvis Y
        ax2 = plt.subplot(6,3,3)# Pelvis Z
        ax3 = plt.subplot(6,3,4)# Hip X
        ax4 = plt.subplot(6,3,5)# Hip Y
        ax5 = plt.subplot(6,3,6)# Hip Z
        ax6 = plt.subplot(6,3,7)# Knee X
        ax7 = plt.subplot(6,3,8)# Knee Y
        ax8 = plt.subplot(6,3,9)# Knee Z
        ax9 = plt.subplot(6,3,10)# Ankle X
        ax10 = plt.subplot(6,3,11)# Ankle Z
        ax11 = plt.subplot(6,3,12)# Footprogress Z
        ax12 = plt.subplot(6,3,13)# ForeFoot X
        ax13 = plt.subplot(6,3,14)# ForeFoot Z
        ax14 = plt.subplot(6,3,15)# ForeFoot Y

        for ax in self.fig.axes:
            ax.set_ylabel("angle (deg)",size=8)
            ax.tick_params(axis='x', which='major', labelsize=6)
            ax.tick_params(axis='y', which='major', labelsize=6)

        ax12.set_xlabel("Cycle %",size=8)
        ax13.set_xlabel("Cycle %",size=8)
        ax14.set_xlabel("Cycle %",size=8)

        if self.m_bodyPart == enums.BodyPartPlot.LowerLimb:
            ax0.set_title("Pelvis Tilt" ,size=8)
            ax1.set_title("Pelvis Obliquity" ,size=8)
            ax2.set_title("Pelvis Rotation" ,size=8)
            ax3.set_title("Hip Flexion" ,size=8)
            ax4.set_title("Hip Adduction" ,size=8)
            ax5.set_title("Hip Rotation" ,size=8)
            ax6.set_title("Knee Flexion" ,size=8)
            ax7.set_title("Knee Adduction" ,size=8)
            ax8.set_title("Knee Rotation" ,size=8)
            ax9.set_title("Ankle dorsiflexion" ,size=8)
            ax10.set_title("Ankle supination" ,size=8)
            ax11.set_title("Foot Progression " ,size=8)
            ax12.set_title("ForeFoot dorsiflexion " ,size=8)
            ax13.set_title("ForeFoot eversion " ,size=8)
            ax14.set_title("ForeFoot Adduction " ,size=8)

            if not self.m_automaticYlim_flag:
                ax0.set_ylim([0,60])
                ax1.set_ylim([-30,30])
                ax2.set_ylim([-30,30])

                ax3.set_ylim( [-20,70])
                ax4.set_ylim([-30,30])
                ax5.set_ylim([-30,30])

                ax6.set_ylim([-15,75])
                ax7.set_ylim([-30,30])
                ax8.set_ylim([-30,30])

                ax9.set_ylim([-30,30])
                ax10.set_ylim([-30,30])
                ax11.set_ylim([-30,30])

                ax12.set_ylim([-50,30])
                ax13.set_ylim([-30,30])
                ax14.set_ylim([-30,30])

        if self.m_bodyPart == enums.BodyPartPlot.Trunk:

            ax0.set_title("Pelvis Tilt" ,size=8)
            ax1.set_title("Pelvis Obliquity" ,size=8)
            ax2.set_title("Pelvis Rotation" ,size=8)
            ax3.set_title("Spine Flexion" ,size=8)
            ax4.set_title("Spine Adduction" ,size=8)
            ax5.set_title("Spine Rotation" ,size=8)
            ax6.set_title("Thorax Anteversion" ,size=8)
            ax7.set_title("Thorax Obliquity" ,size=8)
            ax8.set_title("Thorax Rotation" ,size=8)

            ax9.set_title("Neck Forward Tilt" ,size=8)
            ax10.set_title("Neck Lateral Tilt" ,size=8)
            ax11.set_title("Neck Rotation" ,size=8)

            ax12.set_title("Head Forward Tilt" ,size=8)
            ax13.set_title("Head Lateral Tilt" ,size=8)
            ax14.set_title("Head Rotation" ,size=8)

            if not self.m_automaticYlim_flag:
                ax0.set_ylim([0,60])
                ax1.set_ylim([-30,30])
                ax2.set_ylim([-30,30])

                ax3.set_ylim( [-30,30])
                ax4.set_ylim([-30,30])
                ax5.set_ylim([-30,30])

                ax6.set_ylim([-30,30])
                ax7.set_ylim([-30,30])
                ax8.set_ylim([-30,30])

        if self.m_bodyPart == enums.BodyPartPlot.UpperLimb:

            ax0.set_title("Shoulder flexion" ,size=8)
            ax1.set_title("Shoulder Adduction" ,size=8)
            ax2.set_title("Shoulder Rotation" ,size=8)
            ax3.set_title("Elbow Flexion" ,size=8)
            # ax4.set_title("Elbow Adduction" ,size=8)
            # ax5.set_title("Spine Rotation" ,size=8)
            ax6.set_title("Ulnar Deviation" ,size=8)
            ax7.set_title("Wrist Extension" ,size=8)
            ax8.set_title("Wrist Rotation" ,size=8)

            if not self.m_automaticYlim_flag:
                ax0.set_ylim([-60,60])
                ax1.set_ylim([-30,30])
                ax2.set_ylim([-30,30])

                ax3.set_ylim( [-60,60])
                ax4.set_ylim([-30,30])
                ax5.set_ylim([-30,30])

                ax6.set_ylim([-30,30])
                ax7.set_ylim([-30,30])
                ax8.set_ylim([0,160])

                ax9.set_ylim([-30,30])
                ax10.set_ylim([-30,30])
                ax11.set_ylim([-30,30])

                ax12.set_ylim([-50,30])
                ax13.set_ylim([-30,30])
                ax14.set_ylim([-30,30])

    def setNormativeDataset(self,iNormativeDataSet):
        pass

    def __setData(self):


        if self.m_bodyPart == enums.BodyPartPlot.LowerLimb:
            plot.temporalPlot(self.fig.axes[0],self.m_acq,
                                    "LPelvisAngles",0,pointLabelSuffix=self.m_pointLabelSuffix,color="red")
            plot.temporalPlot(self.fig.axes[1],self.m_acq,
                            "LPelvisAngles",1,pointLabelSuffix=self.m_pointLabelSuffix,color="red")
            plot.temporalPlot(self.fig.axes[2],self.m_acq,
                                    "LPelvisAngles",2,pointLabelSuffix=self.m_pointLabelSuffix,color="red")
            plot.temporalPlot(self.fig.axes[3],self.m_acq,
                                    "LHipAngles",0,pointLabelSuffix=self.m_pointLabelSuffix,color="red")
            plot.temporalPlot(self.fig.axes[4],self.m_acq,
                                    "LHipAngles",1,pointLabelSuffix=self.m_pointLabelSuffix,color="red")
            plot.temporalPlot(self.fig.axes[5],self.m_acq,
                                    "LHipAngles",2,pointLabelSuffix=self.m_pointLabelSuffix,color="red")
            plot.temporalPlot(self.fig.axes[6],self.m_acq,
                                    "LKneeAngles",0,pointLabelSuffix=self.m_pointLabelSuffix,color="red")
            plot.temporalPlot(self.fig.axes[7],self.m_acq,
                                    "LKneeAngles",1,pointLabelSuffix=self.m_pointLabelSuffix,color="red")
            plot.temporalPlot(self.fig.axes[8],self.m_acq,
                                    "LKneeAngles",2,pointLabelSuffix=self.m_pointLabelSuffix,color="red")
            plot.temporalPlot(self.fig.axes[9],self.m_acq,
                                    "LAnkleAngles",0,pointLabelSuffix=self.m_pointLabelSuffix,color="red")
            plot.temporalPlot(self.fig.axes[10],self.m_acq,
                                    "LAnkleAngles",1,pointLabelSuffix=self.m_pointLabelSuffix,color="red")
            plot.temporalPlot(self.fig.axes[11],self.m_acq,
                                    "LFootProgressAngles",2,pointLabelSuffix=self.m_pointLabelSuffix,color="red")
            plot.temporalPlot(self.fig.axes[12],self.m_acq,
                                    "LForeFootAngles",0,pointLabelSuffix=self.m_pointLabelSuffix,color="red")
            plot.temporalPlot(self.fig.axes[13],self.m_acq,
                                    "LForeFootAngles",1,pointLabelSuffix=self.m_pointLabelSuffix,color="red")
            plot.temporalPlot(self.fig.axes[14],self.m_acq,
                                    "LForeFootAngles",2,pointLabelSuffix=self.m_pointLabelSuffix,color="red")


            plot.temporalPlot(self.fig.axes[0],self.m_acq,
                                    "RPelvisAngles",0,pointLabelSuffix=self.m_pointLabelSuffix,color="blue")
            plot.temporalPlot(self.fig.axes[1],self.m_acq,
                            "RPelvisAngles",1,pointLabelSuffix=self.m_pointLabelSuffix,color="blue")
            plot.temporalPlot(self.fig.axes[2],self.m_acq,
                                    "RPelvisAngles",2,pointLabelSuffix=self.m_pointLabelSuffix,color="blue")
            plot.temporalPlot(self.fig.axes[3],self.m_acq,
                                    "RHipAngles",0,pointLabelSuffix=self.m_pointLabelSuffix,color="blue")
            plot.temporalPlot(self.fig.axes[4],self.m_acq,
                                    "RHipAngles",1,pointLabelSuffix=self.m_pointLabelSuffix,color="blue")
            plot.temporalPlot(self.fig.axes[5],self.m_acq,
                                    "RHipAngles",2,pointLabelSuffix=self.m_pointLabelSuffix,color="blue")
            plot.temporalPlot(self.fig.axes[6],self.m_acq,
                                    "RKneeAngles",0,pointLabelSuffix=self.m_pointLabelSuffix,color="blue")
            plot.temporalPlot(self.fig.axes[7],self.m_acq,
                                    "RKneeAngles",1,pointLabelSuffix=self.m_pointLabelSuffix,color="blue")
            plot.temporalPlot(self.fig.axes[8],self.m_acq,
                                    "RKneeAngles",2,pointLabelSuffix=self.m_pointLabelSuffix,color="blue")
            plot.temporalPlot(self.fig.axes[9],self.m_acq,
                                    "RAnkleAngles",0,pointLabelSuffix=self.m_pointLabelSuffix,color="blue")
            plot.temporalPlot(self.fig.axes[10],self.m_acq,
                                    "RAnkleAngles",1,pointLabelSuffix=self.m_pointLabelSuffix,color="blue")
            plot.temporalPlot(self.fig.axes[11],self.m_acq,
                                    "RFootProgressAngles",2,pointLabelSuffix=self.m_pointLabelSuffix,color="blue")
            plot.temporalPlot(self.fig.axes[12],self.m_acq,
                                    "RForeFootAngles",0,pointLabelSuffix=self.m_pointLabelSuffix,color="blue")
            plot.temporalPlot(self.fig.axes[13],self.m_acq,
                                    "RForeFootAngles",1,pointLabelSuffix=self.m_pointLabelSuffix,color="blue")
            plot.temporalPlot(self.fig.axes[14],self.m_acq,
                                    "RForeFootAngles",2,pointLabelSuffix=self.m_pointLabelSuffix,color="blue")

        if self.m_bodyPart == enums.BodyPartPlot.Trunk:

            plot.temporalPlot(self.fig.axes[0],self.m_acq,
                                    "LPelvisAngles",0,pointLabelSuffix=self.m_pointLabelSuffix,color="red")
            plot.temporalPlot(self.fig.axes[1],self.m_acq,
                            "LPelvisAngles",1,pointLabelSuffix=self.m_pointLabelSuffix,color="red")
            plot.temporalPlot(self.fig.axes[2],self.m_acq,
                                    "LPelvisAngles",2,pointLabelSuffix=self.m_pointLabelSuffix,color="red")
            plot.temporalPlot(self.fig.axes[3],self.m_acq,
                                    "LSpineAngles",0,pointLabelSuffix=self.m_pointLabelSuffix,color="red")
            plot.temporalPlot(self.fig.axes[4],self.m_acq,
                                    "LSpineAngles",1,pointLabelSuffix=self.m_pointLabelSuffix,color="red")
            plot.temporalPlot(self.fig.axes[5],self.m_acq,
                                    "LSpineAngles",2,pointLabelSuffix=self.m_pointLabelSuffix,color="red")
            plot.temporalPlot(self.fig.axes[6],self.m_acq,
                                    "LThoraxAngles",0,pointLabelSuffix=self.m_pointLabelSuffix,color="red")
            plot.temporalPlot(self.fig.axes[7],self.m_acq,
                                    "LThoraxAngles",1,pointLabelSuffix=self.m_pointLabelSuffix,color="red")
            plot.temporalPlot(self.fig.axes[8],self.m_acq,
                                    "LThoraxAngles",2,pointLabelSuffix=self.m_pointLabelSuffix,color="red")

            plot.temporalPlot(self.fig.axes[9],self.m_acq,
                                    "LNeckAngles",0,pointLabelSuffix=self.m_pointLabelSuffix,color="red")
            plot.temporalPlot(self.fig.axes[10],self.m_acq,
                                    "LNeckAngles",1,pointLabelSuffix=self.m_pointLabelSuffix,color="red")
            plot.temporalPlot(self.fig.axes[11],self.m_acq,
                                    "LNeckAngles",2,pointLabelSuffix=self.m_pointLabelSuffix,color="red")
            plot.temporalPlot(self.fig.axes[12],self.m_acq,
                                    "LHeadAngles",0,pointLabelSuffix=self.m_pointLabelSuffix,color="red")
            plot.temporalPlot(self.fig.axes[13],self.m_acq,
                                    "LHeadAngles",1,pointLabelSuffix=self.m_pointLabelSuffix,color="red")
            plot.temporalPlot(self.fig.axes[14],self.m_acq,
                                    "LHeadAngles",2,pointLabelSuffix=self.m_pointLabelSuffix,color="red")



            plot.temporalPlot(self.fig.axes[0],self.m_acq,
                                    "RPelvisAngles",0,pointLabelSuffix=self.m_pointLabelSuffix,color="blue")
            plot.temporalPlot(self.fig.axes[1],self.m_acq,
                                    "RPelvisAngles",1,pointLabelSuffix=self.m_pointLabelSuffix,color="blue")
            plot.temporalPlot(self.fig.axes[2],self.m_acq,
                                    "RPelvisAngles",2,pointLabelSuffix=self.m_pointLabelSuffix,color="blue")
            plot.temporalPlot(self.fig.axes[3],self.m_acq,
                                    "RSpineAngles",0,pointLabelSuffix=self.m_pointLabelSuffix,color="blue")
            plot.temporalPlot(self.fig.axes[4],self.m_acq,
                                    "RSpineAngles",1,pointLabelSuffix=self.m_pointLabelSuffix,color="blue")
            plot.temporalPlot(self.fig.axes[5],self.m_acq,
                                    "RSpineAngles",2,pointLabelSuffix=self.m_pointLabelSuffix,color="blue")
            plot.temporalPlot(self.fig.axes[6],self.m_acq,
                                    "RThoraxAngles",0,pointLabelSuffix=self.m_pointLabelSuffix,color="blue")
            plot.temporalPlot(self.fig.axes[7],self.m_acq,
                                    "RThoraxAngles",1,pointLabelSuffix=self.m_pointLabelSuffix,color="blue")
            plot.temporalPlot(self.fig.axes[8],self.m_acq,
                                    "RThoraxAngles",2,pointLabelSuffix=self.m_pointLabelSuffix,color="blue")

            plot.temporalPlot(self.fig.axes[9],self.m_acq,
                                    "RNeckAngles",0,pointLabelSuffix=self.m_pointLabelSuffix,color="blue")
            plot.temporalPlot(self.fig.axes[10],self.m_acq,
                                    "RNeckAngles",1,pointLabelSuffix=self.m_pointLabelSuffix,color="blue")
            plot.temporalPlot(self.fig.axes[11],self.m_acq,
                                    "RNeckAngles",2,pointLabelSuffix=self.m_pointLabelSuffix,color="blue")
            plot.temporalPlot(self.fig.axes[12],self.m_acq,
                                    "RHeadAngles",0,pointLabelSuffix=self.m_pointLabelSuffix,color="blue")
            plot.temporalPlot(self.fig.axes[13],self.m_acq,
                                    "RHeadAngles",1,pointLabelSuffix=self.m_pointLabelSuffix,color="blue")
            plot.temporalPlot(self.fig.axes[14],self.m_acq,
                                    "RHeadAngles",2,pointLabelSuffix=self.m_pointLabelSuffix,color="blue")


        if self.m_bodyPart == enums.BodyPartPlot.UpperLimb:

            plot.temporalPlot(self.fig.axes[0],self.m_acq,
                                    "LShoulderAngles",0,pointLabelSuffix=self.m_pointLabelSuffix,color="red")
            plot.temporalPlot(self.fig.axes[1],self.m_acq,
                            "LShoulderAngles",1,pointLabelSuffix=self.m_pointLabelSuffix,color="red")
            plot.temporalPlot(self.fig.axes[2],self.m_acq,
                                    "LShoulderAngles",2,pointLabelSuffix=self.m_pointLabelSuffix,color="red")
            plot.temporalPlot(self.fig.axes[3],self.m_acq,
                                    "LElbowAngles",0,pointLabelSuffix=self.m_pointLabelSuffix,color="red")
            # plot.temporalPlot(self.fig.axes[4],self.m_acq,
            #                         "LElbowAngles",1,pointLabelSuffix=self.m_pointLabelSuffix,color="red")
            # plot.temporalPlot(self.fig.axes[5],self.m_acq,
            #                         "LElbowAngles",2,pointLabelSuffix=self.m_pointLabelSuffix,color="red")
            plot.temporalPlot(self.fig.axes[6],self.m_acq,
                                    "LWristAngles",0,pointLabelSuffix=self.m_pointLabelSuffix,color="red")
            plot.temporalPlot(self.fig.axes[7],self.m_acq,
                                    "LWristAngles",1,pointLabelSuffix=self.m_pointLabelSuffix,color="red")
            plot.temporalPlot(self.fig.axes[8],self.m_acq,
                                    "LWristAngles",2,pointLabelSuffix=self.m_pointLabelSuffix,color="red")


            plot.temporalPlot(self.fig.axes[0],self.m_acq,
                                    "RShoulderAngles",0,pointLabelSuffix=self.m_pointLabelSuffix,color="blue")
            plot.temporalPlot(self.fig.axes[1],self.m_acq,
                                    "RShoulderAngles",1,pointLabelSuffix=self.m_pointLabelSuffix,color="blue")
            plot.temporalPlot(self.fig.axes[2],self.m_acq,
                                    "RShoulderAngles",2,pointLabelSuffix=self.m_pointLabelSuffix,color="blue")
            plot.temporalPlot(self.fig.axes[3],self.m_acq,
                                    "RElbowAngles",0,pointLabelSuffix=self.m_pointLabelSuffix,color="blue")
            # plot.temporalPlot(self.fig.axes[4],self.m_acq,
            #                         "RElbowAngles",1,pointLabelSuffix=self.m_pointLabelSuffix,color="blue")
            # plot.temporalPlot(self.fig.axes[5],self.m_acq,
            #                         "RElbowAngles",2,pointLabelSuffix=self.m_pointLabelSuffix,color="blue")
            plot.temporalPlot(self.fig.axes[6],self.m_acq,
                                    "RWristAngles",0,pointLabelSuffix=self.m_pointLabelSuffix,color="blue")
            plot.temporalPlot(self.fig.axes[7],self.m_acq,
                                    "RWristAngles",1,pointLabelSuffix=self.m_pointLabelSuffix,color="blue")
            plot.temporalPlot(self.fig.axes[8],self.m_acq,
                                    "RWristAngles",2,pointLabelSuffix=self.m_pointLabelSuffix,color="blue")



    def plotPanel(self):

        self.__setLayer()
        self.__setData()

        # normative dataset not implemented

        return self.fig


class NormalizedKineticsPlotViewer(AbstractPlotViewer):
    """



    """

    def __init__(self,iAnalysis,pointLabelSuffix=None,bodyPart=enums.BodyPartPlot.LowerLimb):

        """
            :Parameters:
                 - `iAnalysis` (pyCGM2.Processing.analysis.Analysis ) - Analysis instance built from AnalysisFilter
                 - `pointLabelSuffix` (str) - suffix ending conventional kinetic CGM labels
    """


        super(NormalizedKineticsPlotViewer, self).__init__(iAnalysis)

        self.m_analysis = self.m_input
        if isinstance(self.m_analysis,pyCGM2.Processing.analysis.Analysis):
            pass
        else:
            logging.error( "[pyCGM2] error input object type. must be a pyCGM2.Core.Processing.analysis.Analysis")

        self.m_bodyPart = bodyPart
        self.m_pointLabelSuffix = pointLabelSuffix
        self.m_normativeData = None
        self.m_flagConsistencyOnly = False
        self.m_concretePlotFunction = None

    def setConcretePlotFunction(self, concreteplotFunction):
        self.m_concretePlotFunction = concreteplotFunction

    def __setLayer(self):

        self.fig = plt.figure(figsize=(8.27,11.69), dpi=100,facecolor="white")
        if self.m_concretePlotFunction.func_name in ["descriptivePlot","gaitDescriptivePlot"]:
            title=u""" Descriptive Time-normalized Kinetics \n """
        elif self.m_concretePlotFunction.func_name in ["consistencyPlot","gaitConsistencyPlot"]:
            title=u""" Consistency Time-normalized Kinetics \n """
        else :
            title=u"""\n"""
        self.fig.suptitle(title)
        plt.subplots_adjust(left=None, bottom=None, right=None, top=None, wspace=0.5, hspace=0.5)


        ax0 = plt.subplot(3,4,1)# Hip X extensor
        ax1 = plt.subplot(3,4,2)# Hip Y abductor
        ax2 = plt.subplot(3,4,3)# Hip Z rotation
        ax3 = plt.subplot(3,4,4)# Knee Z power

        ax4 = plt.subplot(3,4,5)# knee X extensor
        ax5 = plt.subplot(3,4,6)# knee Y abductor
        ax6 = plt.subplot(3,4,7)# knee Z rotation
        ax7 = plt.subplot(3,4,8)# knee Z power

        ax8 = plt.subplot(3,4,9)# ankle X plantar flexion
        ax9 = plt.subplot(3,4,10)# ankle Y rotation
        ax10 = plt.subplot(3,4,11)# ankle Z everter
        ax11 = plt.subplot(3,4,12)# ankle Z power

        for ax in self.fig.axes:
            ax.tick_params(axis='x', which='major', labelsize=6)
            ax.tick_params(axis='y', which='major', labelsize=6)

        for ax in [self.fig.axes[0],self.fig.axes[1],self.fig.axes[2],
                   self.fig.axes[4],self.fig.axes[5],self.fig.axes[6],
                   self.fig.axes[8],self.fig.axes[9],self.fig.axes[10]]:
            ax.set_ylabel("moment (N.mm.kg-1)",size=8)

        for ax in [self.fig.axes[3],self.fig.axes[7],self.fig.axes[8]]:
            ax.set_ylabel("power (W.Kg-1)",size=8)

        ax8.set_xlabel("Cycle %",size=8)
        ax9.set_xlabel("Cycle %",size=8)
        ax10.set_xlabel("Cycle %",size=8)
        ax11.set_xlabel("Cycle %",size=8)

        if self.m_bodyPart == enums.BodyPartPlot.LowerLimb:

            ax0.set_title("Hip extensor Moment" ,size=8)
            ax1.set_title("Hip abductor Moment" ,size=8)
            ax2.set_title("Hip rotation Moment" ,size=8)
            ax3.set_title("Hip Power" ,size=8)

            ax4.set_title("Knee extensor Moment" ,size=8)
            ax5.set_title("Knee abductor Moment" ,size=8)
            ax6.set_title("Knee rotation Moment" ,size=8)
            ax7.set_title("Knee Power" ,size=8)

            ax8.set_title("Ankle plantarflexor Moment" ,size=8)
            ax9.set_title("Ankle everter Moment" ,size=8)
            ax10.set_title("Ankle abductor Moment" ,size=8)
            ax11.set_title("Ankle Power " ,size=8)

            if not self.m_automaticYlim_flag:
                ax0.set_ylim([-2.0 *1000.0, 3.0*1000.0])
                ax1.set_ylim([-2.0*1000.0, 1.0*1000.0])
                ax2.set_ylim([-0.5*1000.0, 0.5*1000.0])
                ax3.set_ylim(  [-3.0, 3.0])

                ax4.set_ylim([-1.0*1000.0, 1.0*1000.0])
                ax5.set_ylim([-1.0*1000.0, 1.0*1000.0])
                ax6.set_ylim([-0.5*1000.0, 0.5*1000.0])
                ax7.set_ylim( [-3.0, 3.0])

                ax8.set_ylim([-1.0*1000.0, 3.0*1000.0])
                ax9.set_ylim([-0.5*1000.0, 0.5*1000.0])
                ax10.set_ylim([-0.5*1000.0, 0.5*1000.0])
                ax11.set_ylim( [-2.0, 5.0])
        else:
            raise Exception("Plot panel not implemented yet")


    def setNormativeDataset(self,iNormativeDataSet):
        """
            **Description :** set a normative gait dataset

            :Parameters:
                 - `iNormativeDataSet` (a class of the pyCGM2.Report.normativeDataset module) - normative gait dataset from pyCGM2.Report.normativeDatabaseProcedure module

        """
        # iNormativeDataSet.constructNormativeData()
        self.m_normativeData = iNormativeDataSet.data

    def __setData(self):
        suffixPlus = "_" + self.m_pointLabelSuffix if self.m_pointLabelSuffix is not None else ""

        if self.m_bodyPart == enums.BodyPartPlot.LowerLimb:
            # hip
            self.m_concretePlotFunction(self.fig.axes[0],self.m_analysis.kineticStats,
                    "LHipMoment"+suffixPlus,"Left",0, color="red", customLimits=None)
            self.m_concretePlotFunction(self.fig.axes[0],self.m_analysis.kineticStats,
                    "RHipMoment"+suffixPlus,"Right",0, color="blue", customLimits=None)

            self.m_concretePlotFunction(self.fig.axes[1],self.m_analysis.kineticStats,
                    "LHipMoment"+suffixPlus,"Left",1, color="red", customLimits=None)
            self.m_concretePlotFunction(self.fig.axes[1],self.m_analysis.kineticStats,
                    "RHipMoment"+suffixPlus,"Right",1, color="blue", customLimits=None)

            self.m_concretePlotFunction(self.fig.axes[2],self.m_analysis.kineticStats,
                    "LHipMoment"+suffixPlus,"Left",2, color="red", customLimits=None)
            self.m_concretePlotFunction(self.fig.axes[2],self.m_analysis.kineticStats,
                    "RHipMoment"+suffixPlus,"Right",2, color="blue", customLimits=None)

            self.m_concretePlotFunction(self.fig.axes[3],self.m_analysis.kineticStats,
                    "LHipPower"+suffixPlus,"Left",2, color="red", customLimits=None)
            self.m_concretePlotFunction(self.fig.axes[3],self.m_analysis.kineticStats,
                    "RHipPower"+suffixPlus,"Right",2, color="blue", customLimits=None)

            # knee
            self.m_concretePlotFunction(self.fig.axes[4],self.m_analysis.kineticStats,
                    "LKneeMoment"+suffixPlus,"Left",0, color="red", customLimits=None)
            self.m_concretePlotFunction(self.fig.axes[4],self.m_analysis.kineticStats,
                    "RKneeMoment"+suffixPlus,"Right",0, color="blue", customLimits=None)

            self.m_concretePlotFunction(self.fig.axes[5],self.m_analysis.kineticStats,
                    "LKneeMoment"+suffixPlus,"Left",1, color="red", customLimits=None)
            self.m_concretePlotFunction(self.fig.axes[5],self.m_analysis.kineticStats,
                    "RKneeMoment"+suffixPlus,"Right",1, color="blue", customLimits=None)

            self.m_concretePlotFunction(self.fig.axes[6],self.m_analysis.kineticStats,
                    "LKneeMoment"+suffixPlus,"Left",2, color="red", customLimits=None)
            self.m_concretePlotFunction(self.fig.axes[6],self.m_analysis.kineticStats,
                    "RKneeMoment"+suffixPlus,"Right",2, color="blue", customLimits=None)

            self.m_concretePlotFunction(self.fig.axes[7],self.m_analysis.kineticStats,
                    "LKneePower"+suffixPlus,"Left",2, color="red", customLimits=None)
            self.m_concretePlotFunction(self.fig.axes[7],self.m_analysis.kineticStats,
                    "RKneePower"+suffixPlus,"Right",2, color="blue", customLimits=None)

            # ankle
            self.m_concretePlotFunction(self.fig.axes[8],self.m_analysis.kineticStats,
                    "LAnkleMoment"+suffixPlus,"Left",0, color="red", customLimits=None)
            self.m_concretePlotFunction(self.fig.axes[8],self.m_analysis.kineticStats,
                    "RAnkleMoment"+suffixPlus,"Right",0, color="blue", customLimits=None)

            self.m_concretePlotFunction(self.fig.axes[9],self.m_analysis.kineticStats,
                    "LAnkleMoment"+suffixPlus,"Left",1, color="red", customLimits=None)
            self.m_concretePlotFunction(self.fig.axes[9],self.m_analysis.kineticStats,
                    "RAnkleMoment"+suffixPlus,"Right",1, color="blue", customLimits=None)

            self.m_concretePlotFunction(self.fig.axes[10],self.m_analysis.kineticStats,
                    "LAnkleMoment"+suffixPlus,"Left",2, color="red", customLimits=None)
            self.m_concretePlotFunction(self.fig.axes[10],self.m_analysis.kineticStats,
                    "RAnkleMoment"+suffixPlus,"Right",2, color="blue", customLimits=None)

            self.m_concretePlotFunction(self.fig.axes[11],self.m_analysis.kineticStats,
                    "LAnklePower"+suffixPlus,"Left",2, color="red", customLimits=None)
            self.m_concretePlotFunction(self.fig.axes[11],self.m_analysis.kineticStats,
                    "RAnklePower"+suffixPlus,"Right",2, color="blue", customLimits=None)

#
    def plotPanel(self):

        if self.m_concretePlotFunction is None:
            raise Exception ("[pyCGM2] need definition of the concrete plot function")

        self.__setLayer()
        self.__setData()

        if self.m_normativeData is not None:
            if self.m_bodyPart == enums.BodyPartPlot.LowerLimb:
                self.fig.axes[0].fill_between(np.linspace(0,100,self.m_normativeData["HipMoment"]["mean"].shape[0]),
                    self.m_normativeData["HipMoment"]["mean"][:,0]-self.m_normativeData["HipMoment"]["sd"][:,0],
                    self.m_normativeData["HipMoment"]["mean"][:,0]+self.m_normativeData["HipMoment"]["sd"][:,0],
                    facecolor="green", alpha=0.5,linewidth=0)

                self.fig.axes[1].fill_between(np.linspace(0,100,self.m_normativeData["HipMoment"]["mean"].shape[0]),
                    self.m_normativeData["HipMoment"]["mean"][:,1]-self.m_normativeData["HipMoment"]["sd"][:,1],
                    self.m_normativeData["HipMoment"]["mean"][:,1]+self.m_normativeData["HipMoment"]["sd"][:,1],
                    facecolor="green", alpha=0.5,linewidth=0)

                self.fig.axes[3].fill_between(np.linspace(0,100,self.m_normativeData["HipPower"]["mean"].shape[0]),
                    self.m_normativeData["HipPower"]["mean"][:,2]-self.m_normativeData["HipPower"]["sd"][:,2],
                    self.m_normativeData["HipPower"]["mean"][:,2]+self.m_normativeData["HipPower"]["sd"][:,2],
                    facecolor="green", alpha=0.5,linewidth=0)


                self.fig.axes[4].fill_between(np.linspace(0,100,self.m_normativeData["KneeMoment"]["mean"].shape[0]),
                    self.m_normativeData["KneeMoment"]["mean"][:,0]-self.m_normativeData["KneeMoment"]["sd"][:,0],
                    self.m_normativeData["KneeMoment"]["mean"][:,0]+self.m_normativeData["KneeMoment"]["sd"][:,0],
                    facecolor="green", alpha=0.5,linewidth=0)

                self.fig.axes[5].fill_between(np.linspace(0,100,self.m_normativeData["KneeMoment"]["mean"].shape[0]),
                    self.m_normativeData["KneeMoment"]["mean"][:,1]-self.m_normativeData["KneeMoment"]["sd"][:,1],
                    self.m_normativeData["KneeMoment"]["mean"][:,1]+self.m_normativeData["KneeMoment"]["sd"][:,1],
                    facecolor="green", alpha=0.5,linewidth=0)

                self.fig.axes[7].fill_between(np.linspace(0,100,self.m_normativeData["KneePower"]["mean"].shape[0]),
                    self.m_normativeData["KneePower"]["mean"][:,2]-self.m_normativeData["KneePower"]["sd"][:,2],
                    self.m_normativeData["KneePower"]["mean"][:,2]+self.m_normativeData["KneePower"]["sd"][:,2],
                    facecolor="green", alpha=0.5,linewidth=0)



                self.fig.axes[8].fill_between(np.linspace(0,100,self.m_normativeData["AnkleMoment"]["mean"].shape[0]),
                    self.m_normativeData["AnkleMoment"]["mean"][:,0]-self.m_normativeData["AnkleMoment"]["sd"][:,0],
                    self.m_normativeData["AnkleMoment"]["mean"][:,0]+self.m_normativeData["AnkleMoment"]["sd"][:,0],
                    facecolor="green", alpha=0.5,linewidth=0)

                self.fig.axes[9].fill_between(np.linspace(0,100,self.m_normativeData["AnkleMoment"]["mean"].shape[0]),
                    self.m_normativeData["AnkleMoment"]["mean"][:,1]-self.m_normativeData["AnkleMoment"]["sd"][:,1],
                    self.m_normativeData["AnkleMoment"]["mean"][:,1]+self.m_normativeData["AnkleMoment"]["sd"][:,1],
                    facecolor="green", alpha=0.5,linewidth=0)


                self.fig.axes[11].fill_between(np.linspace(0,100,self.m_normativeData["AnklePower"]["mean"].shape[0]),
                    self.m_normativeData["AnklePower"]["mean"][:,2]-self.m_normativeData["AnklePower"]["sd"][:,2],
                    self.m_normativeData["AnklePower"]["mean"][:,2]+self.m_normativeData["AnklePower"]["sd"][:,2],
                    facecolor="green", alpha=0.5,linewidth=0)

        return self.fig





class TemporalKineticsPlotViewer(AbstractPlotViewer):
    """


    """

    def __init__(self,iAcq,pointLabelSuffix=None,bodyPart=enums.BodyPartPlot.LowerLimb):

        """
            :Parameters:
                 - `pointLabelSuffix` (str) - suffix ending conventional kinetic CGM labels
        """


        super(TemporalKineticsPlotViewer, self).__init__(iAcq)

        self.m_acq = self.m_input
        if isinstance(self.m_input,pyCGM2.btk.btkAcquisition):
            pass
        else:
            logging.error( "[pyCGM2] error input object type. must be a ma.Trial")

        self.m_pointLabelSuffix = pointLabelSuffix
        self.m_normativeData = None

        self.m_bodyPart = bodyPart

    def __setLayer(self):

        self.fig = plt.figure(figsize=(8.27,11.69), dpi=100,facecolor="white")
        title=u""" Descriptive Time-normalized Kinetics \n """
        self.fig.suptitle(title)
        plt.subplots_adjust(left=None, bottom=None, right=None, top=None, wspace=0.5, hspace=0.5)

        ax0 = plt.subplot(3,4,1)# Hip X extensor
        ax1 = plt.subplot(3,4,2)# Hip Y abductor
        ax2 = plt.subplot(3,4,3)# Hip Z rotation
        ax3 = plt.subplot(3,4,4)# Knee Z power

        ax4 = plt.subplot(3,4,5)# knee X extensor
        ax5 = plt.subplot(3,4,6)# knee Y abductor
        ax6 = plt.subplot(3,4,7)# knee Z rotation
        ax7 = plt.subplot(3,4,8)# knee Z power

        ax8 = plt.subplot(3,4,9)# ankle X plantar flexion
        ax9 = plt.subplot(3,4,10)# ankle Y rotation
        ax10 = plt.subplot(3,4,11)# ankle Z everter
        ax11 = plt.subplot(3,4,12)# ankle Z power

        for ax in self.fig.axes:
            ax.tick_params(axis='x', which='major', labelsize=6)
            ax.tick_params(axis='y', which='major', labelsize=6)


        for ax in [self.fig.axes[0],self.fig.axes[1],self.fig.axes[2],
                   self.fig.axes[4],self.fig.axes[5],self.fig.axes[0],
                   self.fig.axes[8],self.fig.axes[9],self.fig.axes[10]]:
            ax.set_ylabel("moment (N.mm.kg-1)",size=8)

        for ax in [self.fig.axes[3],self.fig.axes[7],self.fig.axes[8]]:
            ax.set_ylabel("power (W.Kg-1)",size=8)

        if self.m_bodyPart == enums.BodyPartPlot.LowerLimb:
            ax0.set_title("Hip extensor Moment" ,size=8)
            ax1.set_title("Hip abductor Moment" ,size=8)
            ax2.set_title("Hip rotation Moment" ,size=8)
            ax3.set_title("Hip Power" ,size=8)

            ax4.set_title("Knee extensor Moment" ,size=8)
            ax5.set_title("Knee abductor Moment" ,size=8)
            ax6.set_title("Knee rotation Moment" ,size=8)
            ax7.set_title("Knee Power" ,size=8)

            ax8.set_title("Ankle plantarflexor Moment" ,size=8)
            ax9.set_title("Ankle everter Moment" ,size=8)
            ax10.set_title("Ankle abductor Moment" ,size=8)
            ax11.set_title("Ankle Power " ,size=8)



            ax9.set_xlabel("Frame %",size=8)
            ax10.set_xlabel("Frame %",size=8)
            ax11.set_xlabel("Frame %",size=8)

            if not self.m_automaticYlim_flag:
                ax0.set_ylim([-2.0 *1000.0, 3.0*1000.0])
                ax1.set_ylim([-2.0*1000.0, 1.0*1000.0])
                ax2.set_ylim([-0.5*1000.0, 0.5*1000.0])
                ax3.set_ylim(  [-3.0, 3.0])

                ax4.set_ylim([-1.0*1000.0, 1.0*1000.0])
                ax5.set_ylim([-1.0*1000.0, 1.0*1000.0])
                ax6.set_ylim([-0.5*1000.0, 0.5*1000.0])
                ax7.set_ylim( [-3.0, 3.0])

                ax8.set_ylim([-1.0*1000.0, 3.0*1000.0])
                ax9.set_ylim([-0.5*1000.0, 0.5*1000.0])
                ax10.set_ylim([-0.5*1000.0, 0.5*1000.0])
                ax11.set_ylim( [-2.0, 5.0])
        else:
            raise Exception ("Plot Panel not implemented yet")

    def setNormativeDataset(self,iNormativeDataSet):
        """
            **Description :** set a normative gait dataset

            :Parameters:
                 - `iNormativeDataSet` (a class of the pyCGM2.Report.normativeDataset module) - normative gait dataset from pyCGM2.Report.normativeDatabaseProcedure module

        """
        pass

    def __setData(self):

        if self.m_bodyPart == enums.BodyPartPlot.LowerLimb:
            plot.temporalPlot(self.fig.axes[0],self.m_acq,
                                    "LHipMoment",0,pointLabelSuffix=self.m_pointLabelSuffix,color="red")
            plot.temporalPlot(self.fig.axes[1],self.m_acq,
                            "LHipMoment",1,pointLabelSuffix=self.m_pointLabelSuffix,color="red")
            plot.temporalPlot(self.fig.axes[2],self.m_acq,
                                    "LHipMoment",2,pointLabelSuffix=self.m_pointLabelSuffix,color="red")
            plot.temporalPlot(self.fig.axes[3],self.m_acq,
                                    "LHipPower",2,pointLabelSuffix=self.m_pointLabelSuffix,color="red")

            plot.temporalPlot(self.fig.axes[4],self.m_acq,
                                    "LKneeMoment",0,pointLabelSuffix=self.m_pointLabelSuffix,color="red")
            plot.temporalPlot(self.fig.axes[5],self.m_acq,
                                    "LKneeMoment",1,pointLabelSuffix=self.m_pointLabelSuffix,color="red")
            plot.temporalPlot(self.fig.axes[6],self.m_acq,
                                    "LKneeMoment",2,pointLabelSuffix=self.m_pointLabelSuffix,color="red")
            plot.temporalPlot(self.fig.axes[7],self.m_acq,
                                    "LKneePower",2,pointLabelSuffix=self.m_pointLabelSuffix,color="red")

            plot.temporalPlot(self.fig.axes[8],self.m_acq,
                                    "LAnkleMoment",0,pointLabelSuffix=self.m_pointLabelSuffix,color="red")
            plot.temporalPlot(self.fig.axes[9],self.m_acq,
                                    "LAnkleMoment",1,pointLabelSuffix=self.m_pointLabelSuffix,color="red")
            plot.temporalPlot(self.fig.axes[10],self.m_acq,
                                    "LAnkleMoment",2,pointLabelSuffix=self.m_pointLabelSuffix,color="red")
            plot.temporalPlot(self.fig.axes[11],self.m_acq,
                                    "LAnklePower",2,pointLabelSuffix=self.m_pointLabelSuffix,color="red")


            plot.temporalPlot(self.fig.axes[0],self.m_acq,
                                    "RHipMoment",0,pointLabelSuffix=self.m_pointLabelSuffix,color="blue")
            plot.temporalPlot(self.fig.axes[1],self.m_acq,
                            "RHipMoment",1,pointLabelSuffix=self.m_pointLabelSuffix,color="blue")
            plot.temporalPlot(self.fig.axes[2],self.m_acq,
                                    "RHipMoment",2,pointLabelSuffix=self.m_pointLabelSuffix,color="blue")
            plot.temporalPlot(self.fig.axes[3],self.m_acq,
                                    "RHipPower",2,pointLabelSuffix=self.m_pointLabelSuffix,color="blue")

            plot.temporalPlot(self.fig.axes[4],self.m_acq,
                                    "RKneeMoment",0,pointLabelSuffix=self.m_pointLabelSuffix,color="blue")
            plot.temporalPlot(self.fig.axes[5],self.m_acq,
                                    "RKneeMoment",1,pointLabelSuffix=self.m_pointLabelSuffix,color="blue")
            plot.temporalPlot(self.fig.axes[6],self.m_acq,
                                    "RKneeMoment",2,pointLabelSuffix=self.m_pointLabelSuffix,color="blue")
            plot.temporalPlot(self.fig.axes[7],self.m_acq,
                                    "RKneePower",2,pointLabelSuffix=self.m_pointLabelSuffix,color="blue")

            plot.temporalPlot(self.fig.axes[8],self.m_acq,
                                    "RAnkleMoment",0,pointLabelSuffix=self.m_pointLabelSuffix,color="blue")
            plot.temporalPlot(self.fig.axes[9],self.m_acq,
                                    "RAnkleMoment",1,pointLabelSuffix=self.m_pointLabelSuffix,color="blue")
            plot.temporalPlot(self.fig.axes[10],self.m_acq,
                                    "RAnkleMoment",2,pointLabelSuffix=self.m_pointLabelSuffix,color="blue")
            plot.temporalPlot(self.fig.axes[11],self.m_acq,
                                    "RAnklePower",2,pointLabelSuffix=self.m_pointLabelSuffix,color="blue")


    def plotPanel(self):

        self.__setLayer()
        self.__setData()

        # normative dataset not implemented

        return self.fig



# -*- coding: utf-8 -*-
#from __future__ import unicode_literals

"""Nexus Operation : **CGM1 Calibration**

:param -l, --leftFlatFoot [int]: enable or disable the flat foot option on the left foot
:param -r, --rightFlatFoot [int]: enable or disable the flat foot option on the right foot
:param -hf, --headFlat [int]: enable or disable the head flat option
:param -md, --markerDiameter [int]: marker diameter
:param -ps, --pointSuffix [string]: suffix adds to the vicon nomenclature outputs
:param --check [bool]: add "cgm1" as point suffix
:param --resetMP [bool]: reset computation of optional parameters, like interAsisDistance, ShankOffsets...

Examples:
    In the script argument box of a python nexus operation, you can edit:

    >>> -l=1 -r=0 -ps=py
    (if you want to add suffix py and enable the flat foot option on the left side only)
    >>> --leftFlatFoot=1 -r=0 --pointSuffix=py --resetMP
    (if you want to add suffix py, enable the flat foot option on the left side only and reset the computation of optional parameters, like interAsisDistance, ShankOffsets...)

"""
#import ipdb
import os
import logging
import argparse

# pyCGM2 settings
import pyCGM2
from pyCGM2 import log; log.setLoggingLevel(logging.INFO)

# vicon nexus
try:
    import ViconNexus
except:
    from viconnexusapi import ViconNexus


# pyCGM2 libraries
from pyCGM2.Utils import files
from pyCGM2.Nexus import nexusFilters, nexusUtils,nexusTools

from pyCGM2.Configurator import CgmArgsManager
from pyCGM2.Lib.CGM import  cgm1


def main():

    parser = argparse.ArgumentParser(description='CGM1 Calibration')
    parser.add_argument('-l','--leftFlatFoot', type=int, help='left flat foot option')
    parser.add_argument('-r','--rightFlatFoot',type=int,  help='right flat foot option')
    parser.add_argument('-hf','--headFlat',type=int,  help='head flat option')
    parser.add_argument('-md','--markerDiameter', type=float, help='marker diameter')
    parser.add_argument('-ps','--pointSuffix', type=str, help='suffix of model outputs')
    parser.add_argument('--check', action='store_true', help='force model output suffix' )
    parser.add_argument('--resetMP', action='store_true', help='reset optional mass parameters')

    args = parser.parse_args()


    NEXUS = ViconNexus.ViconNexus()
    NEXUS_PYTHON_CONNECTED = NEXUS.Client.IsConnected()


    if NEXUS_PYTHON_CONNECTED: # run Operation

        # --------------------------GLOBAL SETTINGS ------------------------------------
        # global setting ( in user/AppData)
        if os.path.isfile(pyCGM2.PYCGM2_APPDATA_PATH + "CGM1-pyCGM2.settings"):
            settings = files.openFile(pyCGM2.PYCGM2_APPDATA_PATH,"CGM1-pyCGM2.settings")
        else:
            settings = files.openFile(pyCGM2.PYCGM2_SETTINGS_FOLDER,"CGM1-pyCGM2.settings")
        # --------------------------CONFIG ------------------------------------
        argsManager = CgmArgsManager.argsManager_cgm1(settings,args)
        leftFlatFoot = argsManager.getLeftFlatFoot()
        rightFlatFoot = argsManager.getRightFlatFoot()
        headFlat = argsManager.getHeadFlat()
        markerDiameter = argsManager.getMarkerDiameter()
        pointSuffix = argsManager.getPointSuffix("cgm1")

        DATA_PATH, calibrateFilenameLabelledNoExt = NEXUS.GetTrialName()

        calibrateFilenameLabelled = calibrateFilenameLabelledNoExt+".c3d"

        logging.info( "data Path: "+ DATA_PATH )
        logging.info( "calibration file: "+ calibrateFilenameLabelled)


        # --------------------------SUBJECT ------------------------------------
        subjects = NEXUS.GetSubjectNames()
        subject = nexusTools.getActiveSubject(NEXUS) #checkActivatedSubject(NEXUS,subjects)
        Parameters = NEXUS.GetSubjectParamNames(subject)


        required_mp,optional_mp = nexusUtils.getNexusSubjectMp(NEXUS,subject,resetFlag=args.resetMP)

        # -------------------------- INFOS ------------------------------------
        mpInfo,mpFilename = files.getMpFileContent(DATA_PATH,"mp.pyCGM2",subject)


        #  translators management
        translators = files.getTranslators(DATA_PATH,"CGM1.translators")
        if not translators:  translators = settings["Translators"]

        # btkAcq builder
        nacf = nexusFilters.NexusConstructAcquisitionFilter(DATA_PATH,calibrateFilenameLabelledNoExt,subject)
        acq = nacf.build()

        # --------------------------MODELLING PROCESSING -----------------------
        model,acqStatic = cgm1.calibrate(DATA_PATH,calibrateFilenameLabelled,translators,
                      required_mp,optional_mp,
                      leftFlatFoot,rightFlatFoot,headFlat,markerDiameter,
                      pointSuffix,forceBtkAcq=acq)


        # ----------------------SAVE-------------------------------------------
        #pyCGM2.model
        files.saveModel(model,DATA_PATH,subject)

        # save mp
        files.saveMp(mpInfo,model,DATA_PATH,mpFilename)

        # ----------------------DISPLAY ON VICON-------------------------------
        nexusUtils.updateNexusSubjectMp(NEXUS,model,subject)
        nexusFilters.NexusModelFilter(NEXUS,
                                      model,acqStatic,subject,
                                      pointSuffix,
                                      staticProcessing=True).run()

        # ========END of the nexus OPERATION if run from Nexus  =========




if __name__ == "__main__":

    main()



# -*- coding: utf-8 -*-
import numpy as np
import logging
import matplotlib.pyplot as plt

from pyCGM2.Utils import files



class NormativeDataGeneratorFilter(object):
    def __init__(self,
                name,modality,
                analyses,
                kinematicLabelsDict=None ,
                kineticLabelsDict =None,
                pointlabelSuffix = None,
                metadata=None):

        self.m_name = name
        self.m_modality = modality
        self.m_analyses = analyses
        self.m_kinematicLabelsDict = kinematicLabelsDict
        self.m_kineticLabelsDict = kineticLabelsDict
        self.m_pointlabelSuffix = pointlabelSuffix
        self.m_metadata =metadata

        self.__normativeData = None
        self.__normalCorridor =  dict()
        self.__allCycles = None

    def getNormativeData(self):
        return self.__normativeData

    def getNormativeCorridor(self):
        return self.__normalCorridor

    def getAllTraces(self):
        return self.__allCycles


    def generate(self):

        cyclesBySide=dict()
        # cycles : ncycle of array(101,3)
        for label in self.m_kinematicLabelsDict["Left"]:
            cyclesBySide[label]=list()
            for analysisIt in self.m_analyses:
                cyclesBySide[label] = cyclesBySide[label]+ analysisIt.kinematicStats.data[label,"Left"]["values"]

        for label in self.m_kinematicLabelsDict["Right"]:
            cyclesBySide[label]=list()
            for analysisIt in self.m_analyses:
                cyclesBySide[label] = cyclesBySide[label]+ analysisIt.kinematicStats.data[label,"Right"]["values"]

        # cycles : ncycle of array(101,3)
        for label in self.m_kineticLabelsDict["Left"]:
            cyclesBySide[label]=list()
            for analysisIt in self.m_analyses:
                cyclesBySide[label] = cyclesBySide[label]+ analysisIt.kineticStats.data[label,"Left"]["values"]

        for label in self.m_kineticLabelsDict["Right"]:
            cyclesBySide[label]=list()
            for analysisIt in self.m_analyses:
                cyclesBySide[label] = cyclesBySide[label]+ analysisIt.kineticStats.data[label,"Right"]["values"]

        # gather both sides
        cycles = dict()
        for key in cyclesBySide.keys():
            new_key = key[1:]
            if new_key in cycles.keys():
                cycles[new_key] = cycles[new_key] + cyclesBySide[key]
            else:
                cycles[new_key] = cyclesBySide[key]

        self.__allCycles = cycles

        out=dict()

        for key in cycles.keys():
            out[key] = dict()
            out[key]["X"] = list()
            out[key]["Y"] = list()
            out[key]["Z"] = list()

            self.__normalCorridor[key] = dict()
            self.__normalCorridor[key]["X"] = list()
            self.__normalCorridor[key]["Y"] = list()
            self.__normalCorridor[key]["Z"] = list()

            X =np.zeros((101,len(cycles[key])))
            Y =np.zeros((101,len(cycles[key])))
            Z =np.zeros((101,len(cycles[key])))

            i=0
            for cycleIt in cycles[key]:
                X[:,i] = cycleIt[:,0]
                Y[:,i] = cycleIt[:,1]
                Z[:,i] = cycleIt[:,2]
                i+=1

            meanX_minusSd = np.mean(X,axis=1) - np.std(X,axis=1)
            meanX_plusSd = np.mean(X,axis=1) + np.std(X,axis=1)

            meanY_minusSd = np.mean(Y,axis=1) - np.std(Y,axis=1)
            meanY_plusSd = np.mean(Y,axis=1) + np.std(Y,axis=1)

            meanZ_minusSd = np.mean(Z,axis=1) - np.std(Z,axis=1)
            meanZ_plusSd = np.mean(Z,axis=1) + np.std(Z,axis=1)

            self.__normalCorridor[key]["X"] = [meanX_minusSd, meanX_plusSd]
            self.__normalCorridor[key]["Y"] = [meanY_minusSd, meanY_plusSd]
            self.__normalCorridor[key]["Z"] = [meanZ_minusSd, meanZ_plusSd]

            for j in range(0,101):
                out[key]["X"].append([j,meanX_minusSd[j], meanX_plusSd[j]])
                out[key]["Y"].append([j,meanY_minusSd[j], meanY_plusSd[j]])
                out[key]["Z"].append([j,meanZ_minusSd[j], meanZ_plusSd[j]])

        self.__normativeData =  {self.m_name :
                                    {self.m_modality :  out},
                                    "metadata": self.m_metadata }

    def save(self,DATA_PATH,filename):
        if self.__normativeData is not None:
            files.saveJson(DATA_PATH, filename, self.__normativeData)
        else:
            raise Exception("[pyCGM2] Normative data is empty")



# -*- coding: utf-8 -*-
import logging
import os
import shutil

import pyCGM2
from pyCGM2.Model.CGM2 import cgm,cgm2
from pyCGM2.Lib.CGM import  cgm2_3
from pyCGM2.Utils import files
from pyCGM2.Utils import utils
from pyCGM2.qtm import qtmTools
from pyCGM2 import enums
from pyCGM2.Tools import btkTools
from  pyCGM2.Lib import eventDetector,report
from pyCGM2.Report import normativeDatasets
import warnings
warnings.simplefilter(action='ignore', category=FutureWarning)
import argparse

from pyCGM2.Inspect import inspectFilters, inspectProcedures

from pyCGM2 import log;
log.setLogger(level = logging.INFO)
with open('pyCGM2.log', 'w'):   pass



MARKERSETS={"Lower limb tracking markers": cgm2.CGM2_3.LOWERLIMB_TRACKING_MARKERS,
            "Thorax tracking markers": cgm.CGM1.THORAX_TRACKING_MARKERS,
            "Upper limb tracking markers": cgm.CGM1.UPPERLIMB_TRACKING_MARKERS,
            "Calibration markers": ["LKNM","RKNM","LMED","RMED","LKAX","LKD1","LKD2","RKAX","RKD1","RKD2"]}


def command():
    parser = argparse.ArgumentParser(description='CGM23 workflow')
    parser.add_argument('--sessionFile', type=str, help='setting xml file from qtm', default="session.xml")

    args = parser.parse_args()
    sessionFilename = args.sessionFile
    main(sessionFilename)


def main(sessionFilename,createPDFReport=True):
    logging.info("------------------------------------------------")
    logging.info("------------QTM - pyCGM2 Workflow---------------")
    logging.info("------------------------------------------------")

    sessionXML = files.readXml(os.getcwd()+"\\",sessionFilename)
    sessionDate = files.getFileCreationDate(os.getcwd()+"\\"+sessionFilename)


    #---------------------------------------------------------------------------
    #management of the Processed folder
    DATA_PATH = os.getcwd()+"\\"+"processed\\"
    files.createDir(DATA_PATH)

    staticMeasurement = qtmTools.findStatic(sessionXML)
    calibrateFilenameLabelled = qtmTools.getFilename(staticMeasurement)
    if not os.path.isfile(DATA_PATH+calibrateFilenameLabelled):
        shutil.copyfile(os.getcwd()+"\\"+calibrateFilenameLabelled,DATA_PATH+calibrateFilenameLabelled)
        logging.info("qualisys exported c3d file [%s] copied to processed folder"%(calibrateFilenameLabelled))

    dynamicMeasurements= qtmTools.findDynamic(sessionXML)
    for dynamicMeasurement in dynamicMeasurements:
        reconstructFilenameLabelled = qtmTools.getFilename(dynamicMeasurement)
        if not os.path.isfile(DATA_PATH+reconstructFilenameLabelled):
            shutil.copyfile(os.getcwd()+"\\"+reconstructFilenameLabelled,DATA_PATH+reconstructFilenameLabelled)
            logging.info("qualisys exported c3d file [%s] copied to processed folder"%(reconstructFilenameLabelled))

            acq=btkTools.smartReader(str(DATA_PATH+reconstructFilenameLabelled))

            acq,zeniState = eventDetector.zeni(acq)

            if zeniState:
                btkTools.smartWriter(acq, str(DATA_PATH + reconstructFilenameLabelled))

                cmd = "Mokka.exe \"%s\""%(str(DATA_PATH + reconstructFilenameLabelled))
                os.system(cmd)


    # --------------------------GLOBAL SETTINGS ------------------------------------
    # global setting ( in user/AppData)

    if os.path.isfile(pyCGM2.PYCGM2_APPDATA_PATH + "CGM2_3-pyCGM2.settings"):
        settings = files.openFile(pyCGM2.PYCGM2_APPDATA_PATH,"CGM2_3-pyCGM2.settings")
    else:
        settings = files.openFile(pyCGM2.PYCGM2_SETTINGS_FOLDER,"CGM2_3-pyCGM2.settings")
    # --------------------------MP ------------------------------------
    required_mp,optional_mp = qtmTools.SubjectMp(sessionXML)

    # --Check MP
    inspectprocedure = inspectProcedures.AnthropometricDataQualityProcedure(required_mp)
    inspector = inspectFilters.QualityFilter(inspectprocedure)
    inspector.run()


    #  translators management
    translators = files.getTranslators(os.getcwd()+"\\","CGM2_3.translators")
    if not translators:  translators = settings["Translators"]

    #  ikweight
    ikWeight = files.getIKweightSet(DATA_PATH,"CGM2_3.ikw")
    if not ikWeight: ikWeight = settings["Fitting"]["Weight"]



    # --------------------------MODEL CALIBRATION -----------------------
    logging.info("--------------------------MODEL CALIBRATION -----------------------")
    staticMeasurement = qtmTools.findStatic(sessionXML)
    calibrateFilenameLabelled = qtmTools.getFilename(staticMeasurement)


    logging.info("----- CALIBRATION-  static file [%s]--"%(calibrateFilenameLabelled))

    leftFlatFoot = utils.toBool(sessionXML.Left_foot_normalised_to_static_trial.text)
    rightFlatFoot = utils.toBool(sessionXML.Right_foot_normalised_to_static_trial.text)
    headFlat = utils.toBool(sessionXML.Head_normalised_to_static_trial.text)
    markerDiameter = float(sessionXML.Marker_diameter.text)*1000.0
    hjcMethod = settings["Calibration"]["HJC"]

    pointSuffix = None

    # Calibration checking
    # --------------------
    acqStatic = btkTools.smartReader(DATA_PATH+calibrateFilenameLabelled)
    for key in MARKERSETS.keys():
        logging.info("[pyCGM2] Checking of the %s"%(key))

        # presence
        ip_presence = inspectProcedures.MarkerPresenceQualityProcedure(acqStatic,
                                        markers = MARKERSETS[key])
        inspector = inspectFilters.QualityFilter(ip_presence)
        inspector.run()

        if ip_presence.markersIn !=[]:

            ip_gap = inspectProcedures.GapQualityProcedure(acqStatic,
                                         markers = ip_presence.markersIn)
            inspector = inspectFilters.QualityFilter(ip_gap)
            inspector.run()

            ip_swap = inspectProcedures.SwappingMarkerQualityProcedure(acqStatic,
                                                markers = ip_presence.markersIn)
            inspector = inspectFilters.QualityFilter(ip_swap)
            inspector.run()

            ip_pos = inspectProcedures.MarkerPositionQualityProcedure(acqStatic,
                                         markers = ip_presence.markersIn)
            inspector = inspectFilters.QualityFilter(ip_pos)

    # Calibration operation
    # --------------------
    logging.info("[pyCGM2] --- calibration operation ---")
    model,acqStatic = cgm2_3.calibrate(DATA_PATH,
        calibrateFilenameLabelled,
        translators,settings,
        required_mp,optional_mp,
        False,
        leftFlatFoot,rightFlatFoot,headFlat,markerDiameter,
        hjcMethod,
        pointSuffix)

    logging.info("----- CALIBRATION-  static file [%s]-----> DONE"%(calibrateFilenameLabelled))

    # --------------------------MODEL FITTING ----------------------------------
    logging.info("--------------------------MODEL FITTING ----------------------------------")
    dynamicMeasurements= qtmTools.findDynamic(sessionXML)

    ik_flag = True

    modelledC3ds = list()
    eventInspectorStates = list()
    for dynamicMeasurement in dynamicMeasurements:

        # reconstructFilenameLabelled = qtmTools.getFilename(dynamicMeasurement)
        reconstructFilenameLabelled = qtmTools.getFilename(dynamicMeasurement)

        logging.info("----Processing of [%s]-----"%(reconstructFilenameLabelled))
        mfpa = qtmTools.getForcePlateAssigment(dynamicMeasurement)
        momentProjection_text = sessionXML.Moment_Projection.text
        if momentProjection_text == "Default":
            momentProjection_text = settings["Fitting"]["Moment Projection"]
        if momentProjection_text == "Distal":
            momentProjection = enums.MomentProjection.Distal
        elif momentProjection_text == "Proximal":
            momentProjection =   enums.MomentProjection.Proximal
        elif momentProjection_text == "Global":
            momentProjection =   enums.MomentProjection.Global
        elif momentProjection_text == "JCS":
            momentProjection =  enums.MomentProjection.JCS



        acq = btkTools.smartReader(DATA_PATH+reconstructFilenameLabelled)

        # Fitting checking
        # --------------------
        for key in MARKERSETS.keys():
            if key != "Calibration markers":

                logging.info("[pyCGM2] Checking of the %s"%(key))
                # presence
                ip_presence = inspectProcedures.MarkerPresenceQualityProcedure(acq,
                                                markers = MARKERSETS[key])
                inspector = inspectFilters.QualityFilter(ip_presence)
                inspector.run()

                if ip_presence.markersIn !=[]:

                    ip_gap = inspectProcedures.GapQualityProcedure(acq,
                                                 markers = ip_presence.markersIn)
                    inspector = inspectFilters.QualityFilter(ip_gap)
                    inspector.run()

                    ip_swap = inspectProcedures.SwappingMarkerQualityProcedure(acq,
                                                        markers = ip_presence.markersIn)
                    inspector = inspectFilters.QualityFilter(ip_swap)
                    inspector.run()

                    ip_pos = inspectProcedures.MarkerPositionQualityProcedure(acq,
                                                 markers = ip_presence.markersIn)
                    inspector = inspectFilters.QualityFilter(ip_pos)


        # filtering
        # -----------------------

        # marker
        order_marker = int(float(dynamicMeasurement.Marker_lowpass_filter_order.text))
        fc_marker = float(dynamicMeasurement.Marker_lowpass_filter_frequency.text)

        # force plate
        order_fp = int(float(dynamicMeasurement.Forceplate_lowpass_filter_order.text))
        fc_fp = float(dynamicMeasurement.Forceplate_lowpass_filter_frequency.text)


        # event checking
        # -----------------------
        inspectprocedureEvents = inspectProcedures.GaitEventQualityProcedure(acq)
        inspector = inspectFilters.QualityFilter(inspectprocedureEvents)
        inspector.run()
        eventInspectorStates.append(inspectprocedureEvents.state)


        # fitting operation
        # -----------------------
        logging.info("[pyCGM2] --- Fitting operation ---")
        acqGait = cgm2_3.fitting(model,DATA_PATH, reconstructFilenameLabelled,
            translators,settings,
            ik_flag,markerDiameter,
            pointSuffix,
            mfpa,
            momentProjection,
            fc_lowPass_marker=fc_marker,
            order_lowPass_marker=order_marker,
            fc_lowPass_forcePlate = fc_fp,
            order_lowPass_forcePlate = order_fp)


        outFilename = reconstructFilenameLabelled
        btkTools.smartWriter(acqGait, str(DATA_PATH + outFilename))
        modelledC3ds.append(outFilename)

        logging.info("----Processing of [%s]-----> DONE"%(reconstructFilenameLabelled))


    # --------------------------GAIT PROCESSING -----------------------
    if not all(eventInspectorStates):
        raise Exception ("[pyCGM2] Impossible to run Gait processing. Badly gait event detection. check the log file")
    logging.info("---------------------GAIT PROCESSING -----------------------")

    if createPDFReport:
        nds = normativeDatasets.Schwartz2008("Free")

        types = qtmTools.detectMeasurementType(sessionXML)
        for type in types:

            modelledTrials = list()
            for dynamicMeasurement in dynamicMeasurements:
                if  qtmTools.isType(dynamicMeasurement,type):
                    filename = qtmTools.getFilename(dynamicMeasurement)
                    modelledTrials.append(filename)

            report.pdfGaitReport(DATA_PATH,model,modelledTrials, nds,pointSuffix, title = type)
            logging.info("----- Gait Processing -----> DONE")



# -*- coding: utf-8 -*-
import argparse
import logging
import os
import pyCGM2
from pyCGM2.Utils import files

MODELS =["CGM1","CGM11", "CGM21",  "CGM22", "CGM23", "CGM24", "CGM25"]

def copyPasteEmgSettings():
    """ copy paste the global emg.settings into the session folder

    :param -m, --model [str] - REQUIRED -:  CGM model (choice is CGM1 CGM11 CGM21  CGM22 CGM23 CGM24 CGM25)

    Examples:

        >>>
        """

    files.copyPaste(pyCGM2.PYCGM2_SETTINGS_FOLDER+"emg.settings",
                    os.getcwd()+"\\"+"emg.settings")

def copyPasteCgmSettings():

    """ copy paste the global CGM#i-pyCGM2.settings into the session folder

    :param -m, --model [str] - REQUIRED -:  CGM model (choice is CGM1 CGM11 CGM21  CGM22 CGM23 CGM24 CGM25)

    Examples:

        >>>
    """

    parser = argparse.ArgumentParser(description='pyCGM2-copyPasteCGMSettings')
    parser.add_argument('-m','--model', type=str, required = True, help="choice is CGM1 CGM11 CGM21  CGM22 CGM23 CGM24 CGM25")

    args = parser.parse_args()

    if args.model not in MODELS:
        raise Exception ("[pyCGM2f] Model not known. Choice is CGM1, CGM11, CGM21.... CGM25")
    else:
        if args.model == "CGM1":
            files.copyPaste(pyCGM2.PYCGM2_SETTINGS_FOLDER+"CGM1-pyCGM2.settings", os.getcwd()+"\\"+"CGM1-pyCGM2.settings")
        if args.model == "CGM11":
            files.copyPaste(pyCGM2.PYCGM2_SETTINGS_FOLDER+"CGM1_1-pyCGM2.settings", os.getcwd()+"\\"+"CGM1_1-pyCGM2.settings")
        if args.model == "CGM21":
            files.copyPaste(pyCGM2.PYCGM2_SETTINGS_FOLDER+"CGM2_1-pyCGM2.settings", os.getcwd()+"\\"+"CGM2_1-pyCGM2.settings")
        if args.model == "CGM22":
            files.copyPaste(pyCGM2.PYCGM2_SETTINGS_FOLDER+"CGM2_2-pyCGM2.settings", os.getcwd()+"\\"+"CGM2_2-pyCGM2.settings")
        if args.model == "CGM23":
            files.copyPaste(pyCGM2.PYCGM2_SETTINGS_FOLDER+"CGM2_3-pyCGM2.settings", os.getcwd()+"\\"+"CGM2_3-pyCGM2.settings")
        if args.model == "CGM24":
            files.copyPaste(pyCGM2.PYCGM2_SETTINGS_FOLDER+"CGM2_4-pyCGM2.settings", os.getcwd()+"\\"+"CGM2_4-pyCGM2.settings")
        if args.model == "CGM25":
            files.copyPaste(pyCGM2.PYCGM2_SETTINGS_FOLDER+"CGM2_5-pyCGM2.settings", os.getcwd()+"\\"+"CGM2_5-pyCGM2.settings")



# -*- coding: utf-8 -*-
import numpy as np
from pyCGM2.Signal import signal_processing
from pyCGM2.Tools import btkTools
from pyCGM2 import enums


class UnithanCoActivationProcedure(object):
    """

    """

    def __init__(self):
        pass

    def run(self,emg1,emg2):


        out = list()
        for c1,c2 in zip(emg1,emg2): # iterate along column
            commonEmg=np.zeros(((101,1)))
            for i in range(0,101):
                commonEmg[i,:]=np.minimum(c1[i],c2[i])
            res=np.trapz(commonEmg,x=np.arange(0,101),axis=0)[0]
            out.append(res)

        return out

class FalconerCoActivationProcedure(object):
    """

    """

    def __init__(self):
        pass

    def run(self,emg1,emg2):


        out = list()
        for c1,c2 in zip(emg1,emg2): # iterate along column

            import matplotlib.pyplot as plt
            plt.plot(c1)
            plt.plot(c2,"r-")

            commonEmg=np.zeros(((101,1)))
            sumEmg = np.zeros(((101,1)))
            for i in range(0,101):
                commonEmg[i,:]=np.minimum(c1[i],c2[i])
                sumEmg[i,:]=c1[i]+c2[i]

            areaNum=np.trapz(commonEmg,x=np.arange(0,101),axis=0)[0]
            areaDen=np.trapz(sumEmg,x=np.arange(0,101),axis=0)[0]
            res = 2.0* areaNum /areaDen
            out.append(res)

            plt.show()
        return out



# -*- coding: utf-8 -*-
import logging
import numpy as np

from pyCGM2 import btk


from  pyCGM2.Math import numeric
from pyCGM2.Tools import  btkTools



class GeneralScoreResidualProcedure(object):

    def __init__(self,model):

        self.m_scoreDefinition = dict()
        self.model = model

    def setDefinition(self, nodeLabel, proxSegLabel, distSegLabel):
        self.m_scoreDefinition[nodeLabel] = {"proximal": proxSegLabel , "distal": distSegLabel }


class ModelScoreResidualProcedure(object):

    def __init__(self, model):

        self.model = model
        self.m_scoreDefinition = dict()

        for it in self.model.m_jointCollection:
            self.m_scoreDefinition[it.m_nodeLabel] = {"proximal": it.m_proximalLabel , "distal": it.m_distalLabel }



class ScoreResidualFilter(object):

    def __init__(self,acq, scoreProcedure):
        self.scoreProcedure = scoreProcedure
        self.acq = acq

        self.m_model = scoreProcedure.model

    def compute(self):

        for nodeLabel in self.scoreProcedure.m_scoreDefinition.keys():

            proxLabel = self.scoreProcedure.m_scoreDefinition[nodeLabel]["proximal"]
            distLabel = self.scoreProcedure.m_scoreDefinition[nodeLabel]["distal"]

            proxNode =  self.m_model.getSegment(proxLabel).anatomicalFrame.getNodeTrajectory(nodeLabel)
            distNode = self.m_model.getSegment(distLabel).anatomicalFrame.getNodeTrajectory(nodeLabel)
            score = numeric.rms((proxNode-distNode),axis = 1)

            scoreValues = np.array([score, np.zeros(self.acq.GetPointFrameNumber()), np.zeros(self.acq.GetPointFrameNumber())]).T

            btkTools.smartAppendPoint(self.acq, str(nodeLabel+"_Score"),scoreValues, PointType=btk.btkPoint.Scalar,desc="Score")



# -*- coding: utf-8 -*-
from __future__ import print_function 
import numpy as np
import logging



def getQuaternionFromMatrix(RotMat):
    """
        Calculates the quaternion representation of the rotation described by RotMat
        Algorithm in Ken Shoemake's article in 1987 SIGGRAPH course notes article "Quaternion Calculus and Fast Animation".

       :Parameters:
           - `RotMat` (numy.array(3,3)) - Rotation Matrix


        :Return:
            - `Quaternion` (numy.array(4)) - 4 components of a quaternion

    """

    Quaternion = np.zeros( (4) );
    Trace = np.trace( RotMat )
    if Trace > 0:
        Root = np.sqrt( Trace + 1 )
        Quaternion[3] = 0.5 * Root
        Root = 0.5 / Root;
        Quaternion[0] = ( RotMat[2,1] - RotMat[1,2] ) * Root
        Quaternion[1] = ( RotMat[0,2] - RotMat[2,0] ) * Root
        Quaternion[2] = ( RotMat[1,0] - RotMat[0,1] ) * Root
    else:
        Next = np.array([ 1, 2, 0 ])
        i = 0
        if RotMat[1,1] > RotMat[0,0]:
            i = 1
        if RotMat[2,2] > RotMat[i,i] :
            i = 2

        j = Next[i]
        k = Next[j]

        Root = np.sqrt( RotMat[i,i] - RotMat[j,j] - RotMat[k,k] + 1 )
        Quaternion[i] = 0.5 * Root
        Root = 0.5 / Root;
        Quaternion[3] = ( RotMat[k,j] - RotMat[j,k] ) * Root
        Quaternion[j] = ( RotMat[j,i] + RotMat[i,j] ) * Root
        Quaternion[k] = ( RotMat[k,i] + RotMat[i,k] ) * Root

    Quaternion = Quaternion / np.linalg.norm( Quaternion)

    return Quaternion


def angleAxisFromQuaternion(Quaternion):
    """
        Calculates the AngleAxis representation of the rotation described by a
        quaternion

       :Parameters:
           - `Quaternion` (numy.array(4)) - 4 components of a quaternion


        :Return:
            - `AngleAxis` (numy.array(3)) - angle Axis in deg

    """

    imag = Quaternion[:-1 ]
    real = Quaternion[ 3 ]

    lenQ = np.linalg.norm( imag )
    if lenQ < 100*np.spacing(np.single(1)):
        AngleAxis = imag
    else:
        angle = 2*np.arctan2( lenQ, real )
        AngleAxis = angle/lenQ * imag

    return np.rad2deg(AngleAxis)

def setFrameData(a1,a2,sequence):
    """
        set Frame of a ccordinate system accoring two vector and a sequence

        :Parameters:
           - `a1` (numy.array(1,3)) - first vector
           - `a2` (str) - second vector
           - `sequence` (str) - construction sequence (XYZ, XYiZ)

        :Return:
            - `axisX` (numy.array(1,3)) - x-axis of the coordinate system
            - `axisY` (numy.array(1,3)) - y-axis of the coordinate system
            - `axisZ` (numy.array(1,3)) - z-axis of the coordinate system
            - `rot` (numy.array(3,3)) - rotation matrix of the coordinate system

        .. note:: if sequence includes a *i* ( ex: XYiZ), opposite of vector a2 is considered

    """

    if sequence == "XYZ" or sequence == "XYiZ" :
        if sequence == "XYiZ":
            a2=a2*-1.0
        axisX=a1
        axisY=a2
        axisZ=np.cross(a1,a2)
        rot=np.array([axisX,axisY,axisZ]).T

    if sequence == "XZY" or sequence == "XZiY" :
        if sequence == "XZiY":
            a2=a2*-1.0
        axisX=a1
        axisZ=a2
        axisY=np.cross(a2,a1)
        rot=np.array([axisX,axisY,axisZ]).T

    if sequence == "YZX" or sequence == "YZiX" :
        if sequence == "YZiX":
            a2=a2*-1.0
        axisY=a1
        axisZ=a2
        axisX=np.cross(a1,a2)
        rot=np.array([axisX,axisY,axisZ]).T

    if sequence == "YXZ" or sequence == "YXiZ" :
        if sequence == "YXiZ":
            a2=a2*-1.0
        axisY=a1
        axisX=a2
        axisZ=np.cross(a2,a1)
        rot=np.array([axisX,axisY,axisZ]).T


    if sequence == "YXZ" or sequence == "YXiZ" :
        if sequence == "YXiZ":
            a2=a2*-1.0
        axisY=a1
        axisX=a2
        axisZ=np.cross(a2,a1)
        rot=np.array([axisX,axisY,axisZ]).T


    if sequence == "ZXY" or sequence == "ZXiY" :
        if sequence == "ZXiY":
            a2=a2*-1.0
        axisZ=a1
        axisX=a2
        axisY=np.cross(a1,a2)
        rot=np.array([axisX,axisY,axisZ]).T

    if sequence == "ZYX" or sequence == "ZYiX" :
        if sequence == "ZYiX":
            a2=a2*-1.0
        axisZ=a1
        axisY=a2
        axisX=np.cross(a2,a1)
        rot=np.array([axisX,axisY,axisZ]).T

    return axisX, axisY, axisZ, rot

class Node(object):
    """
        A node is a local position of a point in a Frame
    """

    def __init__(self,label,desc = ""):
        """
            :Parameters:
               - `label` (str) - desired label of the node

            .. note:: automatically, the suffix "_node" ends the node label
        """

        self.m_name = label+"_node"
        self.m_global = np.zeros((1,3))
        self.m_local = np.zeros((1,3))
        self.m_desc = desc

    def computeLocal(self,rot,t):
        """
            Compute local position from global position

            :Parameters:
                - `rot` (np.array(3,3)) - a rotation matrix
                - `t` (np.array((1,3))) - a translation vector

        """
        self.m_local=np.dot(rot.T,(self.m_global-t))

    def computeGlobal(self,rot,t):
        """
            Compute global position from local

            :Parameters:
                - `rot` (np.array((3,3))) - a rotation matrix
                - `t` (np.array((1,3))) - a translation vector

        """

        self.m_global=np.dot(rot,self.m_local) +t

    def setDescription(self, description):
        self.m_desc = description

    def getLabel(self):
        label = self.m_name
        return label[0:label.find("_node")]


    def getDescription(self):

        return self.m_desc


    def getLocal(self):

        return self.m_local

    def getGlobal(self):

        return self.m_global



class Frame(object):
    """
        A Frame defined a segment pose

    """




    def __init__(self):

        self.m_axisX=np.zeros((1,3))
        self.m_axisY=np.zeros((1,3))
        self.m_axisZ=np.zeros((1,3))

        self._translation=np.zeros((1,3))
        self._matrixRot=np.zeros((3,3))


        self._nodes=[]

    def getRotation(self):
        """
            Get rotation matrix

            :Return:
                - `na` (np.array((3,3))) - a rotation matrix

        """
        return self._matrixRot

    def getAngleAxis(self):

        quaternion = getQuaternionFromMatrix(self._matrixRot)
        axisAngle =  angleAxisFromQuaternion(quaternion)


        return axisAngle


    def getTranslation(self):
        """
            Get translation vector

            :Return:
                - `na` (np.array((3,))) - a translation vector

        """
        return self._translation

    def setRotation(self, R):
        """
            Set rotation matrix

            :Parameters:
               - `R` (np.array(3,3) - a rotation matrix
        """

        self._matrixRot=R

    def setTranslation(self,t):
        """
            Set translation vector

            :Parameters:
               - `t` (np.array(3,)) - a translation vector
        """
        self._translation=t

    def updateAxisFromRotation(self,R):
        """
            Update a rotation matrix

            :Parameters:
               - `R` (np.array(3,3) - a rotation matrix
        """
        self.m_axisX = R[:,0]
        self.m_axisY = R[:,1]
        self.m_axisZ = R[:,2]

        self._matrixRot = R

    def update(self,R,t):
        """
            Update both rotation matrix and translation vector

            :Parameters:
               - `R` (np.array(3,3) - a rotation matrix
               - `t` (np.array(3,)) - a translation vector
        """

        self.m_axisX = R[:,0]
        self.m_axisY = R[:,1]
        self.m_axisZ = R[:,2]
        self._translation = t
        self._matrixRot = R

    def addNode(self,nodeLabel,position, positionType="Global", desc =""):
        """
            Append a `Node` to a Frame

            :Parameters:
                - `nodeLabel` (str) - node label
                - `position` (np.array(3,)) - a translation vector
                - `positionType` (str) - two choice Global or Local

        """
        #TODO : use an Enum for the argment positionType

        logging.debug("new node (%s) added " % nodeLabel)

        isFind=False
        i=0
        for nodeIt in self._nodes:
            if str(nodeLabel+"_node") == nodeIt.m_name:
                isFind=True
                index = i
            i+=1

        if isFind:
            if positionType == "Global":
                self._nodes[index].m_global = position
                self._nodes[index].computeLocal(self._matrixRot,self._translation)
            elif positionType == "Local":
                self._nodes[index].m_local=position
                self._nodes[index].computeGlobal(self._matrixRot,self._translation)
            else :
                raise Exception("positionType not Known (Global or Local")

            logging.debug("[pyCGM2] node (%s) values updated"%(nodeLabel))
            previousDesc = self._nodes[index].m_desc
            if previousDesc != desc:
                logging.debug("[pyCGM2] node (%s) description updated [%s -> %s]"%(nodeLabel, previousDesc, desc))
                self._nodes[index].m_desc = desc

        else:
            node=Node(nodeLabel,desc=desc)
            if positionType == "Global":
                node.m_global=position
                node.computeLocal(self._matrixRot,self._translation)
            elif positionType == "Local":
                node.m_local=position
                node.computeGlobal(self._matrixRot,self._translation)
            else :
                raise Exception("positionType not Known (Global or Local")
            self._nodes.append(node)

    def getNode_byIndex(self,index):
        """
            Return a node within the list from its index

            :Parameters:
                - `index` (int) - index of the node within the list
            :Return:
                - `na` (pyCGM2.pyCGM2.Model.CGM2.Frame.Node) - a node instance
        """
        return self._nodes[index]

    def getNode_byLabel(self,label):
        """
            Return a node in the list from its label

         :Parameters:
            - `label` (str) - label of the node you want to find
         :Return:
            - `na` (pyCGM2.pyCGM2.Model.CGM2.Frame.Node) - a node instance
        """

        for nodeIt in self._nodes:
            if str(label+"_node") == nodeIt.m_name:
                logging.debug( " target label ( %s) - label find (%s) " %(label,nodeIt.m_name) )
                return nodeIt
            #else:
            #    raise Exception("Node label (%s) not found " %(label))

        return False




    def getNodeLabels(self,display=True ):
        """
            Display all node labels

        """
        labels=list()
        for nodeIt in self._nodes:
            labels.append(nodeIt.m_name[:-5])

            if display: print(nodeIt.m_name)

        return labels

    def eraseNodes(self):
        """
            erase all nodes
        """
        self._nodes=[]

    def getNodes(self):

        return self._nodes

    def isNodeExist(self,nodeLabel):

        flag = False
        for nodeIt in self._nodes:
            if str(nodeLabel+"_node") == nodeIt.m_name:
                logging.debug( " target label ( %s) - label find (%s) " %(nodeLabel,nodeIt.m_name) )
                flag = True
                break

        if not flag:
            logging.debug( " node label ( %s) doesn t exist " %(nodeLabel))

        return flag


    def getNodeIndex(self,nodeLabel):

        if self.isNodeExist(nodeLabel):

            for nodeIt in self._nodes:
                if str(nodeLabel+"_node") == nodeIt.m_name:
                    index = i
                    break
                i+=1
            return i
        else:
            raise Exception("[pyCGM2] node label doesn t exist" )


    def updateNode(self,nodeLabel,localArray,globalArray,desc=""):
        """
        update an existing node
        """

        index = self.getNodeIndex(nodeLabel)

        self._nodes[index].m_global = globalArray
        self._nodes[index].m_local = localArray
        self._nodes[index].m_desc = desc

    def copyNode(self,nodeLabel,nodeToCopy):

        indexToCopy = self.getNodeIndex(nodeToCopy)
        globalArray = self._nodes[indexToCopy].m_global
        localArray = self._nodes[indexToCopy].m_local
        desc = self._nodes[indexToCopy].m_desc

        if self.isNodeExist(nodeLabel):
            index =  self.getNodeIndex(nodeToCopy)
            self._nodes[index].m_global = globalArray
            self._nodes[index].m_local = localArray
            self._nodes[index].m_desc = desc
        else:
            self.addNode(nodeLabel,globalArray, position = "Global",desc =  desc)

    def getGlobalPosition(self,nodeLabel):

        node = self.getNode_byLabel(nodeLabel)

        return np.dot(self.getRotation(),node.getLocal())+ self.getTranslation()
